package mahjong;

import java.util.*;

public class Analyser
{
    public Analyser(Player aPlayer)
    {
	player = aPlayer;
	hand = player.getHand();
    }

    /* Funcao que chama as verificaes de batida */
    public boolean analyse(int action, boolean finish)
    {
	this.action = action;
	
	if (hand.isProperlyConfigured())
	{
	    this.tripleRun();       // OK
	    this.closedTsumo();     // OK
	    this.triplePon();       // OK
	    this.allGreen();        // OK
	    this.allEnds();         // OK
	    this.allTerminals();    // OK
	    this.flatHand();        // OK
	    this.allHonors();       // OK
	    this.allSets();         // OK
	    this.insideHand();      // OK
	    this.straight();        // OK
	    this.nineTreasures();   // OK
	
	    this.doubleDoubleRun(); // OK
	    if (!resultado.search(Batida.DOUBLE_DOUBLE_RUN))
	    {
			this.doubleRun();       // OK
			this.allPairs();	
		}
	    
	    this.greaterColors();   // OK
	    if (!resultado.search(Batida.GREATER_COLORS))
	    {
		this.lesserColors();    // OK
		if (!resultado.search(Batida.LESSER_COLORS))
		    this.colors();          // OK
	    }

	    this.fourWinds();       // OK
	    if (!resultado.search(Batida.FOUR_WINDS))
	    {
		this.tableWind();       // OK
		this.seatWind();        // OK
	    }
	    
	    this.fullOutside();     // OK
	    if (!resultado.search(Batida.FULL_OUTSIDE))
		this.halfOutside();     // OK
		    
	    this.fourClosedSets();  // OK
	    if (!resultado.search(Batida.FOUR_CLOSED_SETS))
		this.threeClosedSets(); // OK

	    this.fourQuads();       // OK
	    if (!resultado.search(Batida.FOUR_QUADS))
		this.threeQuads();      // OK

	    this.fullFlush();       // OK
	    if (!resultado.search(Batida.FULL_FLUSH))
		this.halfFlush();       // OK
	}
	else 
	{
	    this.allCrap();         // OK
		this.allPairs();        // OK
	}

	if (resultado.getStatus() || hand.isProperlyConfigured())
	    this.reach();           // OK

	if (resultado.getStatus())
	    this.hasDora();         // OK

	if (resultado.getStatus())  System.out.println("Bateu...");
	else  System.out.println("Nao bateu");
	resultado.printResult();

	if (resultado.getStatus())
	{
	    if (!finish)
		resultado.reset();
	    return true;
	}
	return false;
    }
    
    /* funcoes de cada tipo de batida */
    private void reach()
    {
	if (player.isReach())
	    resultado.insereBatida(new Batida(Batida.REACH, (short) 1));
    } 
    
    private void closedTsumo()
    {
	if ((action == Action.TSUMO) && hand.isClosed())
	    resultado.insereBatida(new Batida(Batida.CLOSED_TSUMO, (short) 1));
    }
    
    private void hasDora()
    {
	int i, j;
	short has;
	OpenMeld aMeld;
	ArrayList list;
	Tile[] doras = Table.getInstance().getDoras();
	
	has = 0;
	list = hand.getClosedTiles();
	for (i = 0; i < doras.length; i++)
	{
	    System.out.println(doras[i]);
	    for (j = 0; j < list.size(); j++)
		if (((Tile) list.get(j)).equals(doras[i].next()))
		    has++;
	}
		
	list = hand.getOpenChi();
	for (i = 0; i < doras.length; i++)
		for (j = 0; j < list.size(); j++)
		{
			aMeld = (OpenMeld) list.get(j);
			if (aMeld.getTiles()[0].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[1].equals(doras[i].next()))
				has++;
			if (aMeld.getStolenTile().equals(doras[i].next()))
				has++;
		}
	list = hand.getOpenPon();
	for (i = 0; i < doras.length; i++)
		for (j = 0; j < list.size(); j++)
	 	{
			aMeld = (OpenMeld) list.get(j);
			if (aMeld.getTiles()[0].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[1].equals(doras[i].next()))
				has++;
			if (aMeld.getStolenTile().equals(doras[i].next()))
				has++;
		}
	list = hand.getOpenKan();
	for (i = 0; i < doras.length; i++)
		for (j = 0; j < list.size(); j++)
		{
			aMeld = (OpenMeld) list.get(j);
			if (aMeld.getTiles()[0].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[1].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[2].equals(doras[i].next()))
				has++;
			if (aMeld.getStolenTile().equals(doras[i].next()))
				has++;
		}
		list = hand.getClosedKan();
	for (i = 0; i < doras.length; i++)
		for (j = 0; j < list.size(); j++)
	 	{
			aMeld = (OpenMeld) list.get(j);
			if (aMeld.getTiles()[0].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[1].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[2].equals(doras[i].next()))
				has++;
			if (aMeld.getTiles()[3].equals(doras[i].next()))
				has++;
			
		}
	
	if (has > 0)
	    resultado.insereBatida(new Batida(Batida.DORA, has));
    }

    private void flatHand()
    {

	if (hand.isClosed() && !hand.hasClosedKan() && hand.isProperlyConfigured(true, false, null))
	{	    
	    boolean isValid = true;
	    for (int i = 0; i < hand.getClosedTiles().size(); i++)
		if (((Tile)hand.getClosedTiles().get(i)).isHonor())
		    isValid = false;

	    if (isValid)
	    {
		// OK: Flat Hand
		Result.getInstance().insereBatida(new Batida(Batida.FLAT_HAND, (short)1));
	    }
	}
    }
    
    private void insideHand()
    {
	int i, j;
	boolean valid = true;
	Tile[] tiles = new Tile[18];
	OpenMeld aMeld;
	ArrayList list;
	
	list = hand.getClosedTiles();
	j = 0;
	for (i = 0; i < list.size() && valid; i++)
	    if (((Tile) list.get(i)).isEnd())
		valid = false;

	if (hand.hasOpenChi() && valid)
	{
	    list = hand.getOpenChi();
	    for (i = 0; i < list.size(); i++, j+=3)
	    {
		aMeld = (OpenMeld) list.get(i);
		tiles[j] = aMeld.getTiles()[0];
		tiles[j+1] = aMeld.getTiles()[1];
		tiles[j+2] = aMeld.getStolenTile();
	    }
	    for (i = 0; tiles[i] != null; i++)
		if (tiles[i].isEnd())
		    valid = false;			
	}
	if (hand.hasOpenPon() && valid)
	{
	    list = hand.getOpenPon();
	    for (i = 0; i < list.size(); i++, j+=3)
	    {
		aMeld = (OpenMeld) list.get(i);
		tiles[j] = aMeld.getTiles()[0];
		tiles[j+1] = aMeld.getTiles()[1];
		tiles[j+2] = aMeld.getStolenTile();
	    }
	    for (i = 0; tiles[i] != null; i+=3)
		if (tiles[i].isEnd())
		    valid = false;			
	}
	if (hand.hasOpenKan() && valid)
	{
	    list = hand.getOpenKan();
	    for (i = 0; i < list.size(); i++, j+=4)
	    {
		aMeld = (OpenMeld) list.get(i);
		tiles[j] = aMeld.getTiles()[0];
		tiles[j+1] = aMeld.getTiles()[1];
		tiles[j+2] = aMeld.getTiles()[2];
		tiles[j+3] = aMeld.getStolenTile();
	    }
	    for (i = 0; tiles[i] != null; i+=3)
		if (tiles[i].isEnd())
		    valid = false;
	}
	if (hand.hasClosedKan() && valid)
	{
	    list = hand.getClosedKan();
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		tiles[j] = aMeld.getTiles()[0];
		tiles[j+1] = aMeld.getTiles()[1];
		tiles[j+2] = aMeld.getTiles()[2];
		tiles[j+3] = aMeld.getTiles()[3];
	    }
	    for (i = 0; tiles[i] != null; i+=4)
		if (tiles[i].isEnd())
		    valid = false;			
	}
	if (valid)
	    resultado.insereBatida(new Batida(Batida.INSIDE_HAND, (short) 1));
    }
    
    private void doubleRun()
    {
	boolean bateu;
	ArrayList pecas = new ArrayList();
	Tile aTile;
	
	bateu = false;
	if (hand.isClosed() && hand.isProperlyConfigured())
	{
	    pecas.addAll(hand.getClosedTiles());
	    for (int i = 0; i < (pecas.size()-5) && !bateu; i++)
	    {
		aTile = (Tile) pecas.get(i);
		if (aTile.equals((Tile) pecas.get(i+1)))
		    for (int j = i+2; j < pecas.size()-3 && !bateu; j++)
			if (((short) aTile.getType() == ((Tile) pecas.get(j)).getType()) &&
			    (((short) aTile.getOrder()+1) == ((Tile) pecas.get(j)).getOrder()) &&
			    ((Tile) pecas.get(j)).equals((Tile) pecas.get(j+1)))
			    for (int k = j+2; k < pecas.size()-1 && !bateu; k++)
				if (((short) aTile.getType() == ((Tile) pecas.get(k)).getType()) &&
				    (((short) aTile.getOrder()+2) == ((Tile)pecas.get(k)).getOrder()) &&
				    ((Tile) pecas.get(k)).equals((Tile) pecas.get(k+1)))
				{
				    hand.removeTile(i);
				    hand.removeTile(i);
				    hand.removeTile(j-2);
				    hand.removeTile(j-2);
				    hand.removeTile(k-4);
				    hand.removeTile(k-4);
				    if (hand.isProperlyConfigured())
			            {
					bateu = true;
					resultado.insereBatida(new Batida(Batida.DOUBLE_RUN, (short) 1));
				    }
				    hand.addClosedTile((Tile) pecas.get(i));
				    hand.addClosedTile((Tile) pecas.get(i+1));
				    hand.addClosedTile((Tile) pecas.get(j));
				    hand.addClosedTile((Tile) pecas.get(j+1));
				    hand.addClosedTile((Tile) pecas.get(k));
				    hand.addClosedTile((Tile) pecas.get(k+1));
				}
	    }
	}
    }
    
    private void straight()
    {
	boolean pinzu, souzu, wanzu;
	int i, j, position;
	Tile[] tiles = new Tile[12];
	OpenMeld aMeld;
	ArrayList listTiles, chis;
	ArrayList removed = new ArrayList();
	
	pinzu = souzu = wanzu = true;
	listTiles = hand.getClosedTiles();
	if (hand.hasOpenChi())
	{
	    chis = hand.getOpenChi();
	    for (i = 0, j = 0; i < chis.size(); i++)
	    {
		aMeld = (OpenMeld) chis.get(i);
		tiles[j] = aMeld.getTiles()[0];
		tiles[j+1] = aMeld.getTiles()[1];
		tiles[j+2] = aMeld.getStolenTile();
	    }
	    for (; j > 0; j--)
		hand.addClosedTile(tiles[j]);
	} 
	
	/* testa sraight para pinzu */	
	for (i = 1; (i < 10) && pinzu; i++)
	{
	    position = listTiles.indexOf(new Tile(TileType.PINZU, (short) i));
	    if (position == -1 )
		pinzu = false;
	    else 
	    {
		hand.removeTile(position);
		removed.add(new Tile(TileType.PINZU, (short) i));
	    }
	}
	if (pinzu && hand.isProperlyConfigured())
	    if (hand.hasOpenChi())
		resultado.insereBatida(new Batida(Batida.STRAIGHT, (short) 1));
	    else
		resultado.insereBatida(new Batida(Batida.STRAIGHT, (short) 2));
 	
	for (i = 0; i < removed.size(); i++)                    // recoloca as pecas removidas
	    hand.addClosedTile((Tile) removed.get(i));
	for (; removed.size() > 0;)                             // esvazia a lista de pecas removidas 
	    removed.remove(0);                                  // mudar para remove all 
	
	/* testa sraight para souzu */	 			
	for (i = 1; (i < 10) && souzu; i++)
	{
	    position = listTiles.indexOf(new Tile(TileType.SOUZU, (short) i));
	    if (position == -1 )
		souzu = false;
	    else 
	    {
		hand.removeTile(position);
		removed.add(new Tile(TileType.SOUZU, (short) i));
	    }
	}
	if (souzu && hand.isProperlyConfigured())
	    if (hand.hasOpenChi())
		resultado.insereBatida(new Batida(Batida.STRAIGHT, (short) 1));
	    else
		resultado.insereBatida(new Batida(Batida.STRAIGHT, (short) 2));
	for (i = 0; i < removed.size(); i++)
	    hand.addClosedTile((Tile) removed.get(i));
	for (; removed.size() > 0;)     
	    removed.remove(0);
	
	/* testa sraight para wanzu */		 			
	for (i = 1; (i < 10) && wanzu; i++)
	{
	    position = listTiles.indexOf(new Tile(TileType.WANZU, (short) i));
	    if (position == -1 )
		wanzu = false;
	    else 
	    {
		hand.removeTile(position);
		removed.add(new Tile(TileType.WANZU, (short) i));
	    }
	}
	if (wanzu && hand.isProperlyConfigured())
	    if (hand.hasOpenChi())
		resultado.insereBatida(new Batida(Batida.STRAIGHT, (short) 1));
	    else
		resultado.insereBatida(new Batida(Batida.STRAIGHT, (short) 2));
	for (i = 0; i < removed.size(); i++)
			hand.addClosedTile((Tile) removed.get(i));
	for (; removed.size() > 0;)     
	    removed.remove(0);	
	
	/* remove as pecas adicionadas pelos CHIs */			 
	for (i = 0; i < tiles.length; i++)                      
	    hand.removeTile(tiles[i]);
    }

    private void tripleRun()
    {
	int i, j, pinzuNext, pinzuNextNext,
	    souzu, souzuNext, souzuNextNext,
	    wanzu, wanzuNext, wanzuNextNext;
	Tile aPinzu, pn, pnn, s, sn, snn, w, wn, wnn;	
	Tile[] tiles = new Tile[12];
	OpenMeld aMeld;
	ArrayList pecas, chis;
	
	pecas = hand.getClosedTiles();
	if (hand.hasOpenChi())
	{
	    chis = hand.getOpenChi();
	    for (i = 0, j = 0; i < chis.size(); i++, j += 3)
	    {
		aMeld = (OpenMeld) chis.get(i); 
		tiles[j+0] = aMeld.getTiles()[0];
		tiles[j+1] = aMeld.getTiles()[1];
		tiles[j+2] = aMeld.getStolenTile();
	    }
	    
	    for (i = 0; i < tiles.length; i++)
	    {
		if (tiles[i] != null)
		    hand.addClosedTile(tiles[i]);
		else
		    break;
	    }
	}
	
	for (i = 0; i < pecas.size()-8; i++)
	{
	    aPinzu = (Tile) pecas.get(i);
	    if ((aPinzu.getType() == TileType.PINZU) && (aPinzu.getOrder() < 8))
	    {
		pinzuNext = pecas.indexOf(new Tile(TileType.PINZU, (short) (aPinzu.getOrder() + 1)));
		pinzuNextNext = pecas.indexOf(new Tile(TileType.PINZU, (short) (aPinzu.getOrder() + 2)));
		souzu = pecas.indexOf(new Tile(TileType.SOUZU, aPinzu.getOrder()));
		souzuNext = pecas.indexOf(new Tile(TileType.SOUZU, (short) (aPinzu.getOrder() + 1)));
		souzuNextNext = pecas.indexOf(new Tile(TileType.SOUZU, (short) (aPinzu.getOrder() + 2)));
		wanzu = pecas.indexOf(new Tile(TileType.WANZU, aPinzu.getOrder()));
		wanzuNext = pecas.indexOf(new Tile(TileType.WANZU, (short) (aPinzu.getOrder() + 1)));
		wanzuNextNext = pecas.indexOf(new Tile(TileType.WANZU, (short) (aPinzu.getOrder() + 2)));
		if ((pinzuNext != -1) && (pinzuNextNext != -1) && 
		    (souzu != -1) && (souzuNext != -1) && (souzuNextNext != -1) &&
		    (wanzu != -1) && (wanzuNext != -1) && (wanzuNextNext != -1))
		{

		    pn = (Tile) pecas.get(pinzuNext);
		    pnn = (Tile) pecas.get(pinzuNextNext);
		    s = (Tile) pecas.get(souzu);
		    sn = (Tile) pecas.get(souzuNext);
		    snn = (Tile) pecas.get(souzuNextNext);
		    w = (Tile) pecas.get(wanzu);
		    wn = (Tile) pecas.get(wanzuNext);
		    wnn = (Tile) pecas.get(wanzuNextNext);
		    
		    hand.removeTile(pecas.indexOf(aPinzu));
		    hand.removeTile(pinzuNext - 1);
		    hand.removeTile(pinzuNextNext - 2);
		    hand.removeTile(souzu - 3);
		    hand.removeTile(souzuNext - 4);
		    hand.removeTile(souzuNextNext - 5);
		    hand.removeTile(wanzu - 6);
		    hand.removeTile(wanzuNext - 7);
		    hand.removeTile(wanzuNextNext - 8);
		    if (hand.isProperlyConfigured())
			if (hand.hasOpenChi())
			    resultado.insereBatida(new Batida(Batida.TRIPLE_RUN, (short) 1));
			else
			    resultado.insereBatida(new Batida(Batida.TRIPLE_RUN, (short) 2)); 
		    hand.addClosedTile(aPinzu);
		    hand.addClosedTile(pn);
		    hand.addClosedTile(pnn);
		    hand.addClosedTile(s);
		    hand.addClosedTile(sn);
		    hand.addClosedTile(snn);
		    hand.addClosedTile(w);
		    hand.addClosedTile(wn);
		    hand.addClosedTile(wnn);
		    break;
		}
	    }
	}
	/* remove as pecas adicionadas pelos CHIs */			 
	for (i = 0; i < tiles.length; i++)                      
	    hand.removeTile(tiles[i]);
    }
    
    private void allSets()
    {
	boolean valid, find;
	short head = 0;
	int i, j;
	Tile aTile;
	Tile[] auxTiles = new Tile[6];
	short[] amount = {0, 0, 0, 0, 0, 0};
	ArrayList tiles;
	
	valid = true;	
	if (!hand.hasOpenChi())
	{
	    tiles = hand.getClosedTiles();
	    for (i = 0; i < tiles.size() && valid; i++)
	    {
		aTile = (Tile) tiles.get(i);
		for (j = 0, find = false; (j < 5) && !find && valid; j++)
		{
		    if (aTile.equals(auxTiles[j]) || (amount[j] == 0))
		    {
			amount[j]++;	
			find = true;
			auxTiles[j] = aTile;
		    }
		    if (amount[5] > 0)
			valid = false;
		}  
	    }
	    for (i = 0; (i < (((tiles.size()-2)/3)+1)) && valid; i++)
	    {
		if (amount[i] == 2)
		    head++;
		if ((amount[i] < 2) || (amount[i] > 3))
		    valid = false;
	    }
	    if (valid && (head == 1))
		resultado.insereBatida(new Batida(Batida.ALL_SETS, (short) 2));
	}	
    }
    
    private void halfOutside()
    {
	// check open chi's, looking for ends
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    OpenMeld m = (OpenMeld)hand.getOpenChi().get(i);
	    if (!m.getTiles()[0].isEnd() &&
		!m.getTiles()[1].isEnd() &&
		!m.getStolenTile().isEnd())
		return;
	}
	
	// check open pon's, looking for ends
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if (!((OpenMeld)hand.getOpenPon().get(i)).getStolenTile().isEnd())
		return;
	System.out.println("pon -> OK");
	
	// check open kan's, looking for ends
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if (!((OpenMeld)hand.getOpenKan().get(i)).getStolenTile().isEnd())
		return;

	// check closed kan's, looking for ends
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	    if (!((OpenMeld)hand.getClosedKan().get(i)).getTiles()[0].isEnd())
		return;

	// cheack the closed tiles, looking for ends
	HalfOutsideChecker checker = new HalfOutsideChecker();
	if (hand.isProperlyConfigured(true, true, checker))
	{
	    // OK: Half Outside
	    if (hand.isClosed())
		Result.getInstance().insereBatida(new Batida(Batida.HALF_OUTSIDE, (short)2));
	    else
		Result.getInstance().insereBatida(new Batida(Batida.HALF_OUTSIDE, (short)1));
	}
    }

    private void triplePon()
    {
	boolean[] suits = new boolean[3];

	for (int order = 1; order <= 9; order++)
	{
	    int i;
	    // no pon's have been found yet
	    for (i = 0; i < suits.length; i++) suits[i] = false;

	    // checking open pon, looking for pon's with the current order
	    OpenMeld m;
	    for (i = 0; i < hand.getOpenPon().size(); i++)
	    {
		m = (OpenMeld)hand.getOpenPon().get(i);
		if (!m.getStolenTile().isHonor() && m.getStolenTile().getOrder() == order)
		    suits[m.getStolenTile().getType()] = true;
	    }

	    // checking open kan, looking for kan's with the current order
	    for (i = 0; i < hand.getOpenKan().size(); i++)
	    {
		m = (OpenMeld)hand.getOpenKan().get(i);
		if (!m.getStolenTile().isHonor() && m.getStolenTile().getOrder() == order)
		    suits[m.getStolenTile().getType()] = true;
	    }

	    // checking closed kan, looking for kan's with the current order
	    for (i = 0; i < hand.getClosedKan().size(); i++)
            {
		m = (OpenMeld)hand.getClosedKan().get(i);
		if (!m.getTiles()[0].isHonor() && m.getTiles()[0].getOrder() == order)
		    suits[m.getTiles()[0].getType()] = true;
	    }

	    // checking closed tiles, looking for sets with the current order...
	    ArrayList gottenTiles = new ArrayList();
	    int idx;
	    int nSame;

	    // ...looking for PINZU set with the current order
	    Tile pinzu = new Tile(TileType.PINZU, (short)order);
	    idx = hand.getClosedTileIdx(pinzu);
	    nSame = 0;
	    if (idx >= 0)
	    {
		while (idx < hand.getClosedTiles().size())
		{
		    if (pinzu.equals((Tile)hand.getClosedTiles().get(idx)))
		    {
			gottenTiles.add((Tile)hand.getClosedTiles().get(idx));
			nSame++;
		    }
		    else
			break;

		    idx++;
		}
		
		if (nSame >= 3) suits[pinzu.getType()] = true;
	    }

	    // ...looking for SOUZU set with the current order
	    Tile souzu = new Tile(TileType.SOUZU, (short)order);
	    idx = hand.getClosedTileIdx(souzu);
	    nSame = 0;
	    if (idx >= 0)
	    {
		while (idx < hand.getClosedTiles().size())
		{
		    if (souzu.equals((Tile)hand.getClosedTiles().get(idx)))
		    {
			gottenTiles.add((Tile)hand.getClosedTiles().get(idx));
			nSame++;
		    }
		    else
			break;

		    idx++;
		}
		
		if (nSame >= 3) suits[souzu.getType()] = true;
	    }
	    
	    // ...looking for WANZU set with the current order
	    Tile wanzu = new Tile(TileType.WANZU, (short)order);
	    idx = hand.getClosedTileIdx(wanzu);
	    nSame = 0;
	    if (idx >= 0)
	    {
		while (idx < hand.getClosedTiles().size())
		{
		    if (wanzu.equals((Tile)hand.getClosedTiles().get(idx)))
		    {
			gottenTiles.add((Tile)hand.getClosedTiles().get(idx));
			nSame++;
		    }
		    else
			break;

		    idx++;
		}
		
		if (nSame >= 3) suits[wanzu.getType()] = true;
	    }

	    // triple pon?
	    if (suits[pinzu.getType()] && suits[souzu.getType()] && suits[wanzu.getType()])
	    {
		for (int k = 0; k < gottenTiles.size(); k++)
		    hand.removeTile((Tile)gottenTiles.get(k));

		if (hand.isProperlyConfigured())
		{
		    System.out.println("order = " + order);

		    // OK: triple pon
		    Result.getInstance().insereBatida(new Batida(Batida.TRIPLE_PON, (short)2));

		    for (int k = 0; k < gottenTiles.size(); k++)
			hand.addClosedTile((Tile)gottenTiles.get(k));
		    break;
		}
		else
		{
		    for (int k = 0; k < gottenTiles.size(); k++)
			hand.addClosedTile((Tile)gottenTiles.get(k));
		}
	    }
	}
    }
    
    private void threeClosedSets()
    {
	ArrayList sets = new ArrayList();
	Tile auxTile;
	int auxIdx;
	int nSame;
	
	int i = 0; 
	while(i < hand.getClosedTiles().size())
	{
	    auxIdx  = i;
	    nSame   = 1;
	    auxTile = (Tile)hand.getClosedTiles().get(auxIdx);

	    i++;
	    while (i < hand.getClosedTiles().size())
	    {
		// count number of same tiles
		if (auxTile.equals((Tile)hand.getClosedTiles().get(i)))
		    nSame++;
		else
		    break;
		i++;
	    }

	    if (nSame >= 3)
            {
		// one set found
		Tile[] set = new Tile[3];
		for (int j = 0; j < 3; j++) set[j] = (Tile)hand.getClosedTiles().get(auxIdx+j);
		sets.add(set);
	    }
	}

	System.out.println("sets.size() = " + sets.size());

	if (sets.size() < 3) return;

	// removing tiles from hand
	for (int j = 0; j < sets.size(); j++)
	{
	    Tile[] tiles = (Tile[])sets.get(j);
	    for (int k = 0; k < tiles.length; k++) hand.removeTile(tiles[k]);
	}

	// testing sets
	if (sets.size() == 3)
	{
	    if (hand.isProperlyConfigured())
	    {
		// OK: Three Closed Sets
		Result.getInstance().insereBatida(new Batida(Batida.THREE_CLOSED_SETS, (short)2));
	    }
	}
	else if (sets.size() == 4)
	{
	    for (int j = 0; j < sets.size(); j++)
	    {
		Tile[] tiles = (Tile[])sets.get(j);
		for (int k = 0; k < tiles.length; k++) hand.addClosedTile(tiles[k]);
		if (hand.isProperlyConfigured())
		{
		    // OK: Three Closed Sets
		    Result.getInstance().insereBatida(new Batida(Batida.THREE_CLOSED_SETS, (short)2));
		    for (int k = 0; k < tiles.length; k++) hand.removeTile(tiles[k]);
		    j = sets.size();
		    break;
		}
	    }
	}

	// returning tiles to the hand
	for (int j = 0; j < sets.size(); j++)
	{
	    Tile[] tiles = (Tile[])sets.get(j);
	    for (int k = 0; k < tiles.length; k++) hand.addClosedTile(tiles[k]);
	}
    }

    private void allPairs()
    {
	int i;
	Tile aTile;	
	boolean valid = true;
	ArrayList listTiles;
	
	if (hand.isClosed() && !(hand.hasClosedKan()))
	{    
	    listTiles = hand.getClosedTiles();
	    for (i = 0; i < listTiles.size() && valid;)
	    {
		aTile = (Tile) listTiles.get(i);
		if (aTile.equals((Tile) listTiles.get(i+1)))
		    i+=2;
		else
		    valid = false;
	    }
	    if (valid)
		resultado.insereBatida(new Batida(Batida.ALL_PAIRS, (short) 2));
	}
    }
    
    private void threeQuads()
    {
	if ((hand.getClosedKan().size() + hand.getOpenKan().size()) >= 3 &&
	    (hand.isProperlyConfigured(true, true, null)))
	{
	    // OK: Four Quads
	    Result.getInstance().insereBatida(new Batida(Batida.THREE_QUADS, (short)2));
	}
    }

    private void halfFlush()
    {
	Hand auxHand = new Hand();

	// gathering the closed tiles
	for (int i = 0; i < hand.getClosedTiles().size(); i++)
	    auxHand.addClosedTile((Tile)hand.getClosedTiles().get(i));

	// gathering the tiles in the open chi's
	OpenMeld m;
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenChi().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getStolenTile());
	}

	// gathering the tiles in the open pon's
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenPon().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getStolenTile());
	}
	
	// gathering the tiles in the open kan's
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenKan().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getTiles()[2]);
	    auxHand.addClosedTile(m.getStolenTile());
	}

	// gathering the tiles in the closed kan's
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	{
	    m = (OpenMeld)hand.getClosedKan().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getTiles()[2]);
	    auxHand.addClosedTile(m.getTiles()[3]);
	}

	Tile firstTile = (Tile)auxHand.getClosedTiles().get(0);

	for (int i = 0; i < auxHand.getClosedTiles().size(); i++)
	    if (((Tile)auxHand.getClosedTiles().get(i)).getType() != firstTile.getType() &&
		!((Tile)auxHand.getClosedTiles().get(i)).isHonor())
		return;

	// OK: Half Flush
	if (hand.isClosed())
	    Result.getInstance().insereBatida(new Batida(Batida.HALF_FLUSH, (short)3));
	else
	    Result.getInstance().insereBatida(new Batida(Batida.HALF_FLUSH, (short)2));	
    }

    private void fullOutside()
    {
	// check open chi's, looking for terminals
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    OpenMeld m = (OpenMeld)hand.getOpenChi().get(i);
	    if (!m.getTiles()[0].isTerminal() &&
		!m.getTiles()[1].isTerminal() &&
		!m.getStolenTile().isTerminal())
		return;
	}
	
	// check open pon's, looking for terminals
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if (!((OpenMeld)hand.getOpenPon().get(i)).getStolenTile().isTerminal())
		return;
	
	// check open kan's, looking for terminals
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if (!((OpenMeld)hand.getOpenKan().get(i)).getStolenTile().isTerminal())
		return;

	// check closed kan's, looking for terminals
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	    if (!((OpenMeld)hand.getClosedKan().get(i)).getTiles()[0].isTerminal())
		return;

	// cheack the closed tiles, looking for terminals
	FullOutsideChecker checker = new FullOutsideChecker();
	if (hand.isProperlyConfigured(true, true, checker))
	{
	    // OK: Full Outside
	    if (hand.isClosed())
		Result.getInstance().insereBatida(new Batida(Batida.FULL_OUTSIDE, (short)3));
	    else
		Result.getInstance().insereBatida(new Batida(Batida.FULL_OUTSIDE, (short)2));
	}
    }

    private void doubleDoubleRun()
    {
	boolean bateu, valid;
	short numDouble;
	int g, i, j, k;
	ArrayList pecas = new ArrayList();
	ArrayList removed = new ArrayList();
	Tile aTile;
	
	valid = true;
	numDouble = 0;
	if (hand.isClosed() && hand.isProperlyConfigured())
	{
	    pecas.addAll(hand.getClosedTiles());
	    for (g = 0; (g < 2) && valid; g++)  
	    {
		bateu = false;
		for (i = 0; i < (pecas.size()-5) && !bateu; i++)
		{
		    aTile = (Tile) pecas.get(i);
		    if (aTile.equals((Tile) pecas.get(i+1)))
			for (j = i+2; j < pecas.size()-3 && !bateu; j++)
			    if (((short) aTile.getType() == ((Tile) pecas.get(j)).getType()) &&
				(((short) aTile.getOrder()+1) == ((Tile) pecas.get(j)).getOrder()) &&
				((Tile) pecas.get(j)).equals((Tile) pecas.get(j+1)))
				for (k = j+2; k < pecas.size()-1 && !bateu; k++)
				    if (((short) aTile.getType() == ((Tile) pecas.get(k)).getType()) &&
					(((short) aTile.getOrder()+2) == ((Tile)pecas.get(k)).getOrder()) &&
					((Tile) pecas.get(k)).equals((Tile) pecas.get(k+1)))
				    {
					System.out.println("g= "+ g +" i= "+ i +" j= "+ j +" k= "+ k);
					System.out.println("size= "+ pecas.size());
					removed.add(pecas.get(i));
					pecas.remove(i);
					hand.removeTile(i);
					
					removed.add(pecas.get(i));
					pecas.remove(i);
					hand.removeTile(i);
					
					removed.add(pecas.get(j-2));
					pecas.remove(j-2);
					hand.removeTile(j-2);
					
					removed.add(pecas.get(j-2));
					pecas.remove(j-2);
					hand.removeTile(j-2);
					
					removed.add(pecas.get(k-4));
					pecas.remove(k-4);
					hand.removeTile(k-4);
					
					removed.add(pecas.get(k-4));
					pecas.remove(k-4);
					hand.removeTile(k-4);
					
					if (hand.isProperlyConfigured())
					{
					    numDouble++;
					    bateu = true;
					}
					else
					    valid = false;
				    }
		    System.out.println("size= "+ pecas.size());
		}
	    }
	    if (numDouble == 2)
		resultado.insereBatida(new Batida(Batida.DOUBLE_DOUBLE_RUN, (short) 3));
	    for (; !removed.isEmpty();) 
		hand.addClosedTile((Tile) removed.remove(0));
	}
    }

    private void lesserColors()
    {
	int[] colors = {0, 0, 0, 0};
	
	// checking open pon, lookin for colors
	OpenMeld m;
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenPon().get(i);
	    if (m.getStolenTile().getType() == TileType.SANGENPAI)
		colors[m.getStolenTile().getOrder()] = 3;
	}

	// checking open kan, looking for colors
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenKan().get(i);
	    if (m.getStolenTile().getType() == TileType.SANGENPAI)
		colors[m.getStolenTile().getOrder()] = 3;
	}

	// checking closed kan, looking for colors
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	{
	    m = (OpenMeld)hand.getClosedKan().get(i);
	    if (m.getTiles()[0].getType() == TileType.SANGENPAI)
		colors[m.getTiles()[0].getOrder()] = 3;
	}

	// checking closed tiles, looking for colors
	int nSame = 0;
	int i = 0;

	// looking for white (haku)
	Tile white = new Tile(TileType.SANGENPAI, (short)1);
	i = hand.getClosedTileIdx(white);
	if (i >= 0)
	{
	    while (i < hand.getClosedTiles().size())
	    {
		if (white.equals((Tile)hand.getClosedTiles().get(i)))
		    nSame++;
		else
		    break;

		i++;
	    }
	    
	    colors[white.getOrder()] = nSame;
	}

	// looking for green (hatsu)
	nSame = 0;
	Tile green = new Tile(TileType.SANGENPAI, (short)2);
	i = hand.getClosedTileIdx(green);
	if (i >= 0)
	{
	    while (i < hand.getClosedTiles().size())
	    {
		if (green.equals((Tile)hand.getClosedTiles().get(i)))
		    nSame++;
		else
		    break;

		i++;
	    }
	    
	    colors[green.getOrder()] = nSame;
	}

	// looking for red (chun)
	nSame = 0;
	Tile red = new Tile(TileType.SANGENPAI, (short)3);
	i = hand.getClosedTileIdx(red);
	if (i >= 0)
	{
	    while (i < hand.getClosedTiles().size())
	    {
		if (red.equals((Tile)hand.getClosedTiles().get(i)))
		    nSame++;
		else
		    break;

		i++;
	    }
	    
	    colors[red.getOrder()] = nSame;
	}

	// lesser colors?
	if ( (colors[1] >= 3 && colors[2] >=3 && colors[3] >= 2) ||
	     (colors[1] >= 3 && colors[2] >=2 && colors[3] >= 3) ||
	     (colors[1] >= 2 && colors[2] >=3 && colors[3] >= 3) )
	{
	    // OK: Lesser Colors
	    Result.getInstance().insereBatida(new Batida(Batida.LESSER_COLORS, (short)4));
	}
    }

    private void allEnds()
    {
	// check open chi's, looking for ends
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    OpenMeld m = (OpenMeld)hand.getOpenChi().get(i);
	    if (!m.getTiles()[0].isEnd() ||
		!m.getTiles()[1].isEnd() ||
		!m.getStolenTile().isTerminal())
		return;
	}

	// check open pon's, looking for ends
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if (!((OpenMeld)hand.getOpenPon().get(i)).getStolenTile().isEnd())
		return;
	
	// check open kan's, looking for ends
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if (!((OpenMeld)hand.getOpenKan().get(i)).getStolenTile().isEnd())
		return;

	// check closed kan's, looking for ends
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	    if (!((OpenMeld)hand.getClosedKan().get(i)).getTiles()[0].isEnd())
		return;	

	// check closed tiles, looking for ends
	for (int i = 0; i < hand.getClosedTiles().size(); i++)
	    if (!((Tile)hand.getClosedTiles().get(i)).isEnd())
		return;

	// OK: all ends
	Result.getInstance().insereBatida(new Batida(Batida.ALL_ENDS, (short)4));
    }

    private void allEndsDiscard()  // ***
    {
	int       i;
	boolean   valid = true;
	ArrayList listTiles;
	
	listTiles = hand.getDiscardedTiles();
	for (i = 0; (i < listTiles.size()) && valid; i++)
	    if (!(((Tile) listTiles.get(i)).isEnd()))
		valid = false;
	if (valid)
	    resultado.insereBatida(new Batida(Batida.ALL_ENDS_DISCARD, (short) 1));	
 	else
	    for(i = 0; i < listTiles.size(); i++)
		if (!((Tile) listTiles.get(i)).isEnd())
		    ((Tile) listTiles.get(i)).addPoints(100);
    }
    
    private void quickOut()
    {
	
    }
    
    private void fullFlush()
    {
	Hand auxHand = new Hand();

	// gathering the closed tiles
	for (int i = 0; i < hand.getClosedTiles().size(); i++)
	    auxHand.addClosedTile((Tile)hand.getClosedTiles().get(i));

	// gathering the tiles in the open chi's
	OpenMeld m;
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenChi().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getStolenTile());
	}

	// gathering the tiles in the open pon's
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenPon().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getStolenTile());
	}
	
	// gathering the tiles in the open kan's
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenKan().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getTiles()[2]);
	    auxHand.addClosedTile(m.getStolenTile());
	}

	// gathering the tiles in the closed kan's
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	{
	    m = (OpenMeld)hand.getClosedKan().get(i);
	    auxHand.addClosedTile(m.getTiles()[0]);
	    auxHand.addClosedTile(m.getTiles()[1]);
	    auxHand.addClosedTile(m.getTiles()[2]);
	    auxHand.addClosedTile(m.getTiles()[3]);
	}

	Tile firstTile = (Tile)auxHand.getClosedTiles().get(0);
	if (firstTile.isHonor()) return;

	for (int i = 0; i < auxHand.getClosedTiles().size(); i++)
	    if (((Tile)auxHand.getClosedTiles().get(i)).getType() != firstTile.getType())
		return;

	// OK: Full Flush
	if (hand.isClosed())
	    Result.getInstance().insereBatida(new Batida(Batida.FULL_FLUSH, (short)6));
	else
	    Result.getInstance().insereBatida(new Batida(Batida.FULL_FLUSH, (short)5));
    }

    private void fourClosedSets()
    {
	if (hand.isClosed() && hand.isProperlyConfigured(false, true, null))
	{
	    // OK: Four Closed Sets
	    Result.getInstance().insereBatida(new Batida(Batida.FOUR_CLOSED_SETS, Batida.LIMIT_HAND));
	}
    }

    private void allCrap()
    {
	int valid, rest, i; 
	boolean[] used = new boolean[Hand.NUM_MAX_CLOSED_TILES]; 
	ArrayList listTiles;
	
	if (!hand.isClosed() || hand.hasClosedKan())
	    valid = -1;
	else
	{
	    listTiles = hand.getClosedTiles();
	    valid = listTiles.indexOf(new Tile(TileType.PINZU, (short) 1));
	    if (valid != -1)
	    {
			used[valid] = true;
			valid = listTiles.indexOf(new Tile(TileType.PINZU, (short) 9));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.SOUZU, (short) 1));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
			valid = listTiles.indexOf(new Tile(TileType.SOUZU, (short) 9));
	    }	
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.WANZU, (short) 1));
	    }
	    if (valid != -1) 
	    {	
			used[valid] = true;
		    valid = listTiles.indexOf(new Tile(TileType.WANZU, (short) 9));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.KAZEHAI, (short) 1));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
			valid = listTiles.indexOf(new Tile(TileType.KAZEHAI, (short) 2));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.KAZEHAI, (short) 3));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
			valid = listTiles.indexOf(new Tile(TileType.KAZEHAI, (short) 4));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.SANGENPAI, (short) 1));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.SANGENPAI, (short) 2));
	    }
	    if (valid != -1) 
	    {
			used[valid] = true;
	    	valid = listTiles.indexOf(new Tile(TileType.SANGENPAI, (short) 3));
	    }
	    if (valid != -1) 
			used[valid] = true;
	    
	    for (i = 0, rest = -1; i  < listTiles.size(); i++)
			if (!used[i])
			{
			    rest = i;
			    break;
			}
	    if ((valid != -1) && (rest >= 0) && (((Tile) listTiles.get(rest)).isEnd()))
			resultado.insereBatida(new Batida(Batida.ALL_CRAP, Batida.LIMIT_HAND));
	}
    }
    
    private void greaterColors()
    {
	boolean[] hasColors = {false, false, false, false};
	int i;
	Tile aTile;
	ArrayList list;
	OpenMeld aMeld;

	list = hand.getClosedTiles();
	for (i = 0; i < list.size()-2; i++)
	{
	    aTile = (Tile) list.get(i);
	    if (aTile.isColor() && aTile.equals(list.get(i+1)) && aTile.equals(list.get(i+2)))
	    {
		hasColors[aTile.getOrder()] = true;
		i+=2;
	    }
	}
	if (hand.hasOpenPon())
	{
	    list = hand.getOpenPon();    
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		aTile = aMeld.getTiles()[0];
		if (aTile.isColor())
		    hasColors[aTile.getOrder()] = true;
	    }
	}	
	if (hand.hasOpenKan())
	{
	    list = hand.getOpenKan();    
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		aTile = aMeld.getTiles()[0];
		if (aTile.isColor())
		    hasColors[aTile.getOrder()] = true;   	   		
	    }
	}	
	if (hand.hasClosedKan())
	{
	    list = hand.getClosedKan();    
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		aTile = aMeld.getTiles()[0];
		if (aTile.isColor())
		    hasColors[aTile.getOrder()] = true;   		
	    }  
	}	 
	if (hasColors[1] && hasColors[2] && hasColors[3])
	    resultado.insereBatida(new Batida(Batida.GREATER_COLORS, Amount.LIMIT_HAND));	
    }
    
    private void fourWinds()
    {
	boolean[] hasWinds = {false, false, false, false, false};
	int i, numWinds;
	Tile aTile;
	ArrayList list;
	OpenMeld aMeld;

	list = hand.getClosedTiles();
	for (i = 0; i < list.size()-2; i++)
	{
	    aTile = (Tile) list.get(i);
	    if (aTile.isWind() && aTile.equals(list.get(i+1)) && aTile.equals(list.get(i+2)))
	    {
		hasWinds[aTile.getOrder()] = true;
		i+=2;
	    }
	}
	if (hand.hasOpenPon())
	{
	    list = hand.getOpenPon();    
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		aTile = aMeld.getTiles()[0];
		if (aTile.isWind())
		    hasWinds[aTile.getOrder()] = true;
	    }
	}	
	if (hand.hasOpenKan())
	{
	    list = hand.getOpenKan();    
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		aTile = aMeld.getTiles()[0];
		if (aTile.isWind())
		    hasWinds[aTile.getOrder()] = true;   	   		
	    }
	}	
	if (hand.hasClosedKan())
	{
	    list = hand.getClosedKan();    
	    for (i = 0; i < list.size(); i++)
	    {
		aMeld = (OpenMeld) list.get(i);
		aTile = aMeld.getTiles()[0];
		if (aTile.isWind())
		    hasWinds[aTile.getOrder()] = true;   		
	    }  
	}	 
	for (i = 0, numWinds = 0; i < hasWinds.length; i++)
	    if (hasWinds[i])
		numWinds++;
	if (numWinds == 3)
	{
	    list = hand.getClosedTiles();
	    for (i = 0; i < list.size()-1; i++)
		if (((Tile) list.get(i)).equals((Tile) list.get(i+1)))
		    hasWinds[((Tile) list.get(i)).getOrder()] = true;
	}
	if (hasWinds[1] && hasWinds[2] && hasWinds[3] && hasWinds[4])
	    resultado.insereBatida(new Batida(Batida.FOUR_WINDS, Amount.LIMIT_HAND));
    }

    private void allGreen()
    {
	// check open chi's, looking for greens
	OpenMeld m;
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    m = (OpenMeld)hand.getOpenChi().get(i);
	    if (!AllGreenChecker.isGreen(m.getTiles()[0]) ||
		!AllGreenChecker.isGreen(m.getTiles()[1]) ||
		!AllGreenChecker.isGreen(m.getStolenTile()))
		return;
	}

	// check open pon's, looking for greens
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if  (!AllGreenChecker.isGreen(((OpenMeld)hand.getOpenPon().get(i)).getStolenTile()))
		return;

	// check open kan's, looking for greens
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if  (!AllGreenChecker.isGreen(((OpenMeld)hand.getOpenKan().get(i)).getStolenTile()))
		return;

	// check closed kan's, looking for greens
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	    if  (!AllGreenChecker.isGreen(((OpenMeld)hand.getClosedKan().get(i)).getTiles()[0]))
		return;

	// check closed tiles, looking for greens
	AllGreenChecker checker = new AllGreenChecker();
	if (hand.isProperlyConfigured(true, true, checker))
	{
	    // OK: All Green
	    Result.getInstance().insereBatida(new Batida(Batida.ALL_GREEN, Batida.LIMIT_HAND));
	}
    }

    private void allHonors()
    {
	ArrayList list;
	int       i;
	boolean   valid = true;
	Tile      aTile;
	Tile[]    tiles;
	OpenMeld  aMeld;
	
	if (hand.hasOpenChi())
	    valid = false;
	else
	{
	    list = hand.getClosedTiles();
	    for (i = 0; (i < list.size()) && valid; i++)
	    {
		aTile = (Tile) list.get(i);
		if (!aTile.isHonor())
		    valid = false;
	    }
	    if (hand.hasOpenPon() && valid)
	    {
		list = hand.getOpenPon();    
		for (i = 0; (i < list.size()) && valid; i++)
		{
		    aMeld = (OpenMeld) list.get(i);
		    tiles = aMeld.getTiles();
		    if (!tiles[0].isHonor())
			valid = false;		   		
		}
	    }	
	    if (hand.hasOpenKan() && valid)
		{
		    list = hand.getOpenKan();    
		    for (i = 0; (i < list.size()) && valid; i++)
		    {
			aMeld = (OpenMeld) list.get(i);
			tiles = aMeld.getTiles();
			if (!tiles[0].isHonor())
			    valid = false;		   		
		    }
	    	}	
	    if (hand.hasClosedKan() && valid)
	    {
		list = hand.getClosedKan();    
		for (i = 0; (i < list.size()) && valid; i++)
		{
		    aMeld = (OpenMeld) list.get(i);
		    tiles = aMeld.getTiles();
		    if (!tiles[0].isHonor())
			valid = false;		    		
		}  
	    }	 
	    if (valid)
		resultado.insereBatida(new Batida(Batida.ALL_HONORS, Amount.LIMIT_HAND));
	}
    }
    
    private void colors()
    {
	ArrayList listTiles;
	int       i;
	Tile      aTile;
	Tile[]    tiles;
	OpenMeld  aMeld;
	
	listTiles = hand.getClosedTiles();
	for (i = 0; i < listTiles.size()-2; i++)
	{
	    aTile = (Tile) listTiles.get(i);
	    if ( aTile.isColor() && aTile.equals((Tile) listTiles.get(i+2)))
	    { 
		resultado.insereBatida(new Batida(Batida.COLORS + " " + aTile.getName(), (short) 1));
		i+=2;
	    }
	}
	if (hand.hasOpenPon())
	{
	    listTiles = hand.getOpenPon();    
	    for (i = 0; i < listTiles.size(); i++)
	    {
		aMeld = (OpenMeld) listTiles.get(i);
		tiles = aMeld.getTiles();
		if (tiles[0].isColor())
		    resultado.insereBatida(new Batida(Batida.COLORS + " " + tiles[0].getName(), (short) 1));
	    }
	}	
	if (hand.hasOpenKan())
	{
	    listTiles = hand.getOpenKan();    
	    for (i = 0; i < listTiles.size(); i++)
	    {
		aMeld = (OpenMeld) listTiles.get(i);
		tiles = aMeld.getTiles();	
		if (tiles[0].isColor())
		    resultado.insereBatida(new Batida(Batida.COLORS + " " + tiles[0].getName(), (short) 1));
	    }
	}		
	if (hand.hasClosedKan())
	{
	    listTiles = hand.getClosedKan();    
	    for (i = 0; i < listTiles.size(); i++)
		{
		    aMeld = (OpenMeld) listTiles.get(i);
		    tiles = aMeld.getTiles();
		    if (tiles[0].isColor())
			resultado.insereBatida(new Batida(Batida.COLORS + " " + tiles[0].getName(), (short)1)); 
		}
	}	
    }			   	
    
    private void tableWind()
    {
	Tile windTile;

	// define the current wind tile
	switch (Table.getInstance().getCurrentRound())
	{
	case (Position.EAST):
	    windTile = new Tile(TileType.KAZEHAI, (short)1);
	    break;
	case (Position.SOUTH):
	    windTile = new Tile(TileType.KAZEHAI, (short)2);
	    break;
	case Position.WEST:
	    windTile = new Tile(TileType.KAZEHAI, (short)3);
	    break;
	case Position.NORTH:
	    windTile = new Tile(TileType.KAZEHAI, (short)4);
	    break;
	default:
	    windTile = new Tile((short)-1, (short)-1); // unknown tile
	    break;
	}
	
	// search for the wind tile among the open pon's
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if (((OpenMeld)hand.getOpenPon().get(i)).getStolenTile().equals(windTile))
            {
		// OK: Table Wind
		Result.getInstance().insereBatida(new Batida(Batida.TABLE_WIND, (short)1));
		return;
	    }

	// search for the wind tile among the open kan's
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if (((OpenMeld)hand.getOpenKan().get(i)).getStolenTile().equals(windTile))
            {
		// OK: Table Wind
		Result.getInstance().insereBatida(new Batida(Batida.TABLE_WIND, (short)1));
		return;
	    }

	// search for the wind tile among the closed kan's
	OpenMeld m;
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	{
	    m = (OpenMeld)hand.getClosedKan().get(i);
	    if (((Tile)m.getTiles()[0]).equals(windTile))
            {
		// OK: Table Wind
		Result.getInstance().insereBatida(new Batida(Batida.TABLE_WIND, (short)1));
		return;
	    }
	}

	// search for the wind tile among the closed tiles
	Tile[] gottenTiles = new Tile[3];
	int i, idx;
	for (i = 0; i < 3; i++)
	{
	    idx = hand.getClosedTileIdx(windTile);
	    if (idx >= 0) 
            {
		gottenTiles[i] = (Tile)hand.getClosedTiles().get(idx);
		hand.removeTile(idx);
	    }
	    else break;
	}

	if ((i == 3) && hand.isProperlyConfigured())
	{
	    // OK: Table Wind
	    Result.getInstance().insereBatida(new Batida(Batida.TABLE_WIND, (short)1));
	}

	for (i = 0; i < gottenTiles.length; i++)
	    if (gottenTiles[i] != null)
		hand.addClosedTile(gottenTiles[i]);

	return;
    }
	
    private void seatWind()
    {
	Tile windTile;

	// define the player wind tile
	switch (player.getPosition())
	{
	case (Position.EAST):
	    windTile = new Tile(TileType.KAZEHAI, (short)1);
	    break;
	case (Position.SOUTH):
	    windTile = new Tile(TileType.KAZEHAI, (short)2);
	    break;
	case Position.WEST:
	    windTile = new Tile(TileType.KAZEHAI, (short)3);
	    break;
	case Position.NORTH:
	    windTile = new Tile(TileType.KAZEHAI, (short)4);
	    break;
	default:
	    windTile = new Tile((short)-1, (short)-1); // unknown tile
	    break;
	}
	
	// search for the wind tile among the open pon's
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if (((OpenMeld)hand.getOpenPon().get(i)).getStolenTile().equals(windTile))
            {
		// OK: Seat Wind
		Result.getInstance().insereBatida(new Batida(Batida.SEAT_WIND, (short)1));
		return;
	    }

	// search for the wind tile among the open kan's
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if (((OpenMeld)hand.getOpenKan().get(i)).getStolenTile().equals(windTile))
            {
		// OK: Seat Wind
		Result.getInstance().insereBatida(new Batida(Batida.SEAT_WIND, (short)1));
		return;
	    }

	// search for the wind tile among the closed kan's
	OpenMeld m;
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	{
	    m = (OpenMeld)hand.getClosedKan().get(i);
	    if (((Tile)m.getTiles()[0]).equals(windTile))
            {
		// OK: Seat Wind
		Result.getInstance().insereBatida(new Batida(Batida.SEAT_WIND, (short)1));
		return;
	    }
	}

	// search for the wind tile among the closed tiles
	Tile[] gottenTiles = new Tile[3];
	int i, idx;
	for (i = 0; i < 3; i++)
	{
	    idx = hand.getClosedTileIdx(windTile);
	    if (idx >= 0) 
            {
		gottenTiles[i] = (Tile)hand.getClosedTiles().get(idx);
		hand.removeTile(idx);
	    }
	    else break;
	}

	if ((i == 3) && hand.isProperlyConfigured())
	{
	    // OK: Seat Wind
	    Result.getInstance().insereBatida(new Batida(Batida.SEAT_WIND, (short)1));
	}
	
	for (i = 0; i < gottenTiles.length; i++)
	    if (gottenTiles[i] != null)
		hand.addClosedTile(gottenTiles[i]);

	return;
    }

    private void allTerminals()
    {
	// check open chi's, looking for terminals
	for (int i = 0; i < hand.getOpenChi().size(); i++)
	{
	    OpenMeld m = (OpenMeld)hand.getOpenChi().get(i);
	    if (!m.getTiles()[0].isTerminal() ||
		!m.getTiles()[1].isTerminal() ||
		!m.getStolenTile().isTerminal())
		return;
	}

	// check open pon's, looking for ends
	for (int i = 0; i < hand.getOpenPon().size(); i++)
	    if (!((OpenMeld)hand.getOpenPon().get(i)).getStolenTile().isTerminal())
		return;
	
	// check open kan's, looking for ends
	for (int i = 0; i < hand.getOpenKan().size(); i++)
	    if (!((OpenMeld)hand.getOpenKan().get(i)).getStolenTile().isTerminal())
		return;

	// check closed kan's, looking for ends
	for (int i = 0; i < hand.getClosedKan().size(); i++)
	    if (!((OpenMeld)hand.getClosedKan().get(i)).getTiles()[0].isTerminal())
		return;	

	// check closed tiles, looking for ends
	for (int i = 0; i < hand.getClosedTiles().size(); i++)
	    if (!((Tile)hand.getClosedTiles().get(i)).isTerminal())
		return;

	// OK: all ends
	Result.getInstance().insereBatida(new Batida(Batida.ALL_TERMINALS, Batida.LIMIT_HAND));
    }

    private void outFromStart()
    {

    }

    private void nineTreasures()
    {
	boolean valid = true;
	int i;
	Tile aTile;
	ArrayList closedTiles = new ArrayList();

	if (hand.isClosed() && !hand.hasClosedKan())
	{	
	    closedTiles.addAll(hand.getClosedTiles());
	    aTile = (Tile) closedTiles.remove(0);
	    if (aTile.isHonor() || (aTile.getOrder() != 1))
		valid = false;
	    for (i = 2; i < 9 && valid; i++)
	    {
		if (!closedTiles.remove(new Tile(aTile.getType(), (short) i)))
		    valid = false;
	    }
	    if (valid)
	    {
		closedTiles.remove(new Tile(aTile.getType(), (short) 1));
		closedTiles.remove(new Tile(aTile.getType(), (short) 1));
		closedTiles.remove(new Tile(aTile.getType(), (short) 9));
		closedTiles.remove(new Tile(aTile.getType(), (short) 9));
		closedTiles.remove(new Tile(aTile.getType(), (short) 9));
	    }
	    if ((closedTiles.size() == 1) && 
		(((Tile) closedTiles.get(0)).getType() == aTile.getType()))
		resultado.insereBatida(new Batida(Batida.NINE_TREASURES, Batida.LIMIT_HAND));
	}
    }

    private void fourQuads()
    {
	if ((hand.getClosedKan().size() + hand.getOpenKan().size()) == 4 &&
	    (hand.isProperlyConfigured(false, false, null)))
	{
	    // OK: Four Quads
	    Result.getInstance().insereBatida(new Batida(Batida.FOUR_QUADS, Batida.LIMIT_HAND));
	}
    }

    private Table mesa = Table.getInstance();
    private int action;
    private Result resultado = Result.getInstance();
    private Hand hand;
    private Player player;
}





