package mahjong;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class CenterPanel extends JPanel
{
    public CenterPanel()
    {
	super();

	Font titleFont = new Font(null, Font.BOLD, 16);
	JLabel roundStringLabel = new JLabel(roundString);
	JLabel stageStringLabel = new JLabel(stageString);
	JLabel tilesStringLabel = new JLabel(tilesString);
	roundStringLabel.setFont(titleFont);
	stageStringLabel.setFont(titleFont);
	tilesStringLabel.setFont(titleFont);
	roundLabel.setFont(titleFont);
	stageLabel.setFont(titleFont);
	tilesLabel.setFont(titleFont);
	
	JPanel infoPanel = new JPanel();
	infoPanel.setLayout(new GridLayout(3, 2, 5, 5));
	infoPanel.add(roundStringLabel);
	infoPanel.add(roundLabel);
	infoPanel.add(stageStringLabel);
	infoPanel.add(stageLabel);
	infoPanel.add(tilesStringLabel);
	infoPanel.add(tilesLabel);
	
	JPanel dorasPanel = new JPanel();
	dorasPanel.setBorder(BorderFactory.createTitledBorder("Dora:"));
	for (int i = 0; i < doras.length; i++)
	{
	    doras[i] = new JLabel();
	    dorasPanel.add(doras[i]);
	}

	JPanel leftPanel = new JPanel();
	leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
	roundImageLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	playerImageLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	arrowImageLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	leftPanel.add(roundImageLabel);
	leftPanel.add(playerImageLabel);
	leftPanel.add(arrowImageLabel);

	JPanel rightPanel = new JPanel();
	rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
	rightPanel.setBorder(BorderFactory.createTitledBorder(""));
	rightPanel.add(infoPanel);
	rightPanel.add(dorasPanel);

	JPanel mainPanel1 = new JPanel();
	mainPanel1.setAlignmentY(Component.CENTER_ALIGNMENT);
	mainPanel1.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	mainPanel1.add(leftPanel);
	mainPanel1.add(rightPanel);

	JPanel mainPanel2 = new JPanel();
	mainPanel2.setBackground(new Color(0.20f, 0.70f, 0.20f));
	mainPanel2.setLayout(new BoxLayout(mainPanel2, BoxLayout.X_AXIS));
	mainPanel2.setAlignmentY(Component.CENTER_ALIGNMENT);
	mainPanel2.add(Box.createHorizontalGlue());
	mainPanel2.add(mainPanel1);
	mainPanel2.add(Box.createHorizontalGlue());

	this.notifyInit(Position.EAST, Position.EAST);
	this.setBackground(new Color(0.20f, 0.70f, 0.20f));
	this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	this.add(Box.createVerticalGlue());
	this.add(mainPanel2);
	this.add(Box.createVerticalGlue());
    }

    public void notifyInit(short dealer, short round)
    {
	// defining round info
	switch (round)
	{
	case Position.EAST:
	    //roundImageLabel.setIcon(ImageServer.getInstance().getImage("east_round"));
	    roundLabel.setText("Leste");
	    break;
	case Position.SOUTH:
	    //roundImageLabel.setIcon(ImageServer.getInstance().getImage("south_round"));
	    roundLabel.setText("Sul");
	    break;
	}

	// defining stage info
	switch(dealer)
	{
	case Position.EAST:
	    stageLabel.setText("" + 1);
	    break;
	case Position.SOUTH:
	    stageLabel.setText("" + 2);
	    break;
	case Position.WEST:
	    stageLabel.setText("" + 3);
	    break;
	case Position.NORTH:
	    stageLabel.setText("" + 4);
	    break;
	}

	// defining remaining tiles info
	tilesLabel.setText("" + 70);
	nTiles = 70;

	// cleaning dora info
	ImageIcon icon = ImageServer.getInstance().getImage("back_0");
	for (int i = 0; i < doras.length; i++) doras[i].setIcon(icon);
	nDoras = 0;

	// cleaning current player info
	playerImageLabel.setIcon(null);
	arrowImageLabel.setIcon(null);
    }

    public void notifyDora(Tile tile)
    {
	if (nDoras < doras.length)
	{
	    doras[nDoras].setIcon(ImageServer.getInstance().getImage(tile.getName() + "_0"));
	    nDoras++;
	}
	else System.out.println("Error adding new dora...");
    }

    public void notifyRemainingTiles(short nTiles)
    {
	this.nTiles = nTiles;
	tilesLabel.setText("" + nTiles);
    }

    public void notifyWhoIsPlaying(short position)
    {
	switch (position)
	{
	case Position.EAST:
	    roundImageLabel.setText("Leste");
	    arrowImageLabel.setIcon(ImageServer.getInstance().getImage("arrow_0"));
	    playerImageLabel.setIcon(null);
	    break;

	case Position.SOUTH:
	    roundImageLabel.setText("Sul");
	    arrowImageLabel.setIcon(ImageServer.getInstance().getImage("arrow_90"));
	    playerImageLabel.setIcon(ImageServer.getInstance().getImage("player_90"));
	    break;

	case Position.WEST:
	    roundImageLabel.setText("Oeste");
	    arrowImageLabel.setIcon(ImageServer.getInstance().getImage("arrow_180"));
	    playerImageLabel.setIcon(ImageServer.getInstance().getImage("player_180"));
	    break;

	case Position.NORTH:
	    roundImageLabel.setText("Norte");
	    arrowImageLabel.setIcon(ImageServer.getInstance().getImage("arrow_270"));
	    playerImageLabel.setIcon(ImageServer.getInstance().getImage("player_270"));
	    break;
	}
    }

    private JLabel doras[] = new JLabel[5];
    private JLabel roundLabel = new JLabel();
    private JLabel stageLabel = new JLabel();
    private JLabel tilesLabel = new JLabel();
    private JLabel roundImageLabel = new JLabel();
    private JLabel arrowImageLabel = new JLabel();
    private JLabel playerImageLabel = new JLabel();
    private short nDoras;
    private short nTiles;
    
    static String roundString = "Rodada: ";
    static String stageString = "Fase: ";
    static String tilesString = "Pedras restantes: ";
}






