package mahjong;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class EastPlayerPanel extends PlayerPanel
{
    public EastPlayerPanel()
    {
	super();
        defaultRotation = 0;

        JPanel auxPanel1 = new JPanel();
        auxPanel1.setLayout(new BoxLayout(auxPanel1, BoxLayout.X_AXIS));
	auxPanel1.setBackground(new Color(0.20f, 0.70f, 0.20f));

	//assembling the discarded tiles panel
        JPanel auxPanel2 = new JPanel();
        auxPanel2.setLayout(new BoxLayout(auxPanel2, BoxLayout.X_AXIS));
	auxPanel2.setBackground(new Color(0.20f, 0.70f, 0.20f));
        discardedTilesPanel.setLayout(new GridLayout(2, 9, 5, 5));
	for (int i = 0; i < discardedTiles.length; i++)
	{
	    discardedTiles[i] = new JLabel();
	    discardedTilesPanel.add(discardedTiles[i]);
	}
        auxPanel2.add(Box.createHorizontalGlue());
        auxPanel2.add(discardedTilesPanel);
        auxPanel2.add(Box.createHorizontalGlue());
        auxPanel1.add(auxPanel2);

        //assembling the buttons panel
        buttonsPanel.setLayout(new GridLayout(0, 1));
	discardButton.setEnabled(false);
	reachButton.setEnabled(false);
	kanButton.setEnabled(false);
        buttonsPanel.add(discardButton);
        buttonsPanel.add(reachButton);
	buttonsPanel.add(kanButton);
        auxPanel1.add(buttonsPanel);

	//assembling the reach panel
	reachPanel.setLayout(new BoxLayout(reachPanel, BoxLayout.X_AXIS));
	reachPanel.add(diceLabel);
	reachPanel.add(Box.createHorizontalGlue());
	reachPanel.add(reachLabel);
	reachPanel.add(Box.createHorizontalGlue());

	// assembling the closed tiles panel
        closedTilesPanel.setLayout(new BoxLayout(closedTilesPanel, BoxLayout.X_AXIS));
	for (int i = 0; i < closedTiles.length; i++)
	{
	    JPanel auxPanel = new JPanel();
	    auxPanel.setLayout(new BoxLayout(auxPanel, BoxLayout.Y_AXIS));
	    auxPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	    selectedTiles[i] = new JLabel();
	    closedTiles[i] = new TaggedLabel(i);
	    selectedTiles[i].setAlignmentX(Component.CENTER_ALIGNMENT);
	    closedTiles[i].setAlignmentX(Component.CENTER_ALIGNMENT);
	    auxPanel.add(closedTiles[i]);
	    auxPanel.add(selectedTiles[i]);
	    closedTilesPanel.add(auxPanel);
	}
        JPanel auxPanel3 = new JPanel();
	auxPanel3.setBackground(new Color(0.20f, 0.70f, 0.20f));
        auxPanel3.add(closedTilesPanel);

	// assembling melds panel
	meldsPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 5));
	for (int i = 0; i < 4; i++)
	{
            melds[i] = new EastMeldPanel();
	    meldsPanel.add(melds[i]);
	}
        auxPanel3.add(meldsPanel);

	makeInitHand();
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        this.add(auxPanel1);
        this.add(reachPanel);
        this.add(auxPanel3);

	// defining the mouse listeners for the closedTiles labels
	MouseAdapter m = new MouseAdapter()
	    {
		public void mouseClicked(MouseEvent e)
		{
		    TaggedLabel label = (TaggedLabel)e.getComponent();
		    
		    if (selectedTile >= 0 && selectedTile < 14)
			selectedTiles[selectedTile].setIcon(null);

		    if (label.getIcon() != null)
		    {
			selectedTile = (short)label.getTag();
			selectedTiles[selectedTile].setIcon(ImageServer.getInstance().getImage("selection"));
		    }
		}
	    };
	
	for (int i = 0; i < 14; i++) closedTiles[i].addMouseListener(m);

	// defining the action listener for the buttons
	discardButton.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if (selectedTile >= 0 && selectedTile < 14)
			if (closedTiles[selectedTile].getIcon() != null)
			{
			    discardButton.setEnabled(false);
			    reachButton.setEnabled(false);
			    kanButton.setEnabled(false);
			    lastAction = UserAction.USER_ACTION_DISCARD;
			    return;
			}

		    lastAction = UserAction.USER_ACTION_NONE;
		}
	    });

	reachButton.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if (selectedTile >= 0 && selectedTile < 14)
			if (closedTiles[selectedTile].getIcon() != null)
			{
			    discardButton.setEnabled(false);
			    reachButton.setEnabled(false);
			    kanButton.setEnabled(false);
			    lastAction = UserAction.USER_ACTION_REACH;
			    return;
			}

		    lastAction = UserAction.USER_ACTION_NONE;
		}
	    });

	kanButton.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if (selectedTile >= 0 && selectedTile < 14)
			if (closedTiles[selectedTile].getIcon() != null)
			{
			    discardButton.setEnabled(false);
			    reachButton.setEnabled(false);
			    kanButton.setEnabled(false);
			    lastAction = UserAction.USER_ACTION_KAN;
			    return;
			}

		    lastAction = UserAction.USER_ACTION_NONE;
		}
	    });

    }

    // overriding updateClosedTilesPanels from PlayerPlanel
    protected void updateClosedTilesPanel(Hand h)
    {
	int i;
	ArrayList tiles = h.getClosedTiles();
	ImageServer imageServer = ImageServer.getInstance();

	for (i = 0; i < tiles.size(); i++)
            closedTiles[i].setIcon(imageServer.getImage(((Tile)tiles.get(i)).getName() + 
							"_" + defaultRotation));

	for (; i < closedTiles.length; i++)
	    closedTiles[i].setIcon(null);

	if (selectedTile >= 0 && selectedTile < 14)
	{
	    selectedTiles[selectedTile].setIcon(null);
	    selectedTile = 0;
	}
	
    }

    public void notifyRealPlayerPlaying()
    {
	lastAction = UserAction.USER_ACTION_NONE;
	discardButton.setEnabled(true);
	reachButton.setEnabled(true);
	kanButton.setEnabled(true);
    }

    public void notifySelectedTile(short tileIdx)
    {
	if (selectedTile >= 0 && selectedTile < 14)
	    selectedTiles[selectedTile].setIcon(null);
	
	selectedTile = tileIdx;
	if (selectedTile >= 0 && selectedTile < 14)
	{
	    if (closedTiles[selectedTile].getIcon() == null) selectedTile = 1;
	    selectedTiles[selectedTile].setIcon(ImageServer.getInstance().getImage("selection"));
	}
    }

    public int getLastAction()
    {
	return (lastAction);
    }

    public int getSelectedTileIdx()
    {
	return (selectedTile);
    }

    // buttons panel
    private JPanel buttonsPanel = new JPanel();
    private JButton discardButton = new JButton("Descartar");
    private JButton reachButton = new JButton("Richi");
    private JButton kanButton = new JButton("Kan");

    // selection label
    private JLabel selectedTiles[] = new JLabel[14];
    private short selectedTile = -1;

    // user action
    private volatile int lastAction;
}

class EastMeldPanel extends MeldPanel
{
    EastMeldPanel()
    {
	super();
        defaultRotation = 0;

	// Assembling panel
        this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        for (int i = 0; i < 4; i++)
	{
	    upperTiles[i] = new JLabel();
	    lowerTiles[i] = new JLabel();
	    //coupleTilePanel[i] = new JPanel();
            coupleTilePanel[i].setAlignmentY(Component.BOTTOM_ALIGNMENT);
            coupleTilePanel[i].setLayout(new BoxLayout(coupleTilePanel[i], BoxLayout.Y_AXIS));
            coupleTilePanel[i].add(upperTiles[i]);
            coupleTilePanel[i].add(lowerTiles[i]);
	    this.add(coupleTilePanel[i]);
	}
    }
}









