package mahjong;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

public class FinishedHandDialog extends JDialog
{
    public FinishedHandDialog(JFrame owner)
    {
	super(owner, "Finished Hand Dialog", true);

	// initializing components
	playerImage = new JLabel();
	playerName = new JLabel();
	closedTiles = new JLabel[18];
	for (int i = 0; i < closedTiles.length; i++) closedTiles[i] = new JLabel();
	lastTile = new JLabel();
	drawType = new JLabel();
	selectedTile = new JLabel();
	openMelds = new EastMeldPanel[4];
	for (int i = 0; i < openMelds.length; i++) openMelds[i] = new EastMeldPanel();
	finishedLabel = new JLabel();
	totalsLabel = new JLabel();
	scoreLabel = new JLabel();
	closeButton = new JButton("Fechar");

	// assembling the hand panel
	JPanel handPanel = new JPanel();
	handPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	handPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED),
							       BorderFactory.createEmptyBorder(10, 10, 10, 10)));
	JPanel lastTilePanel = new JPanel();
	lastTilePanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	lastTilePanel.setLayout(new BoxLayout(lastTilePanel, BoxLayout.Y_AXIS));
	selectedTile.setBackground(new Color(0.20f, 0.70f, 0.20f));
	lastTilePanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	drawType.setAlignmentX(Component.CENTER_ALIGNMENT);
	lastTile.setAlignmentX(Component.CENTER_ALIGNMENT);
	selectedTile.setAlignmentX(Component.CENTER_ALIGNMENT);
	lastTilePanel.add(drawType);
	lastTilePanel.add(lastTile);
	lastTilePanel.add(selectedTile);

	for (int i = 0; i < closedTiles.length; i++)
	    handPanel.add(closedTiles[i]);
	for (int i = 0; i < openMelds.length; i++)
	    handPanel.add(openMelds[i]);
	handPanel.add(lastTilePanel);

	// assembling the list panel
	JPanel listPanel = new JPanel();
	listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.X_AXIS));
	finishedLabel.setAlignmentY(Component.TOP_ALIGNMENT);
	totalsLabel.setAlignmentY(Component.TOP_ALIGNMENT);
	scoreLabel.setAlignmentY(Component.TOP_ALIGNMENT);
	listPanel.add(finishedLabel);
	listPanel.add(Box.createHorizontalGlue());
	listPanel.add(totalsLabel);
	listPanel.add(Box.createHorizontalGlue());
	listPanel.add(scoreLabel);
	
	// assembling the main panel
	playerImage.setAlignmentX(Component.CENTER_ALIGNMENT);
	playerName.setAlignmentX(Component.CENTER_ALIGNMENT);
	handPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
	listPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
	closeButton.setAlignmentX(Component.CENTER_ALIGNMENT);

	JPanel mainPanel = new JPanel();
	mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
	mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	mainPanel.add(playerImage);
	mainPanel.add(playerName);
	mainPanel.add(Box.createRigidArea(new Dimension(0,  5)));
	mainPanel.add(handPanel);
	mainPanel.add(Box.createRigidArea(new Dimension(0, 15)));
	mainPanel.add(listPanel);
	mainPanel.add(Box.createRigidArea(new Dimension(0,  5)));
	mainPanel.add(closeButton);
	this.getContentPane().add(mainPanel);

	// definig the close button aciton
	final FinishedHandDialog thisDialog = this;
	closeButton.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    thisDialog.hide();
		}
	    });
    }

    public void show(Player p, int[] score, int tali, int total)
    {
	// setting up player identification
	if ( p!= null)
	{
	    switch (p.getPosition())
	    {
	    case Position.EAST:
		playerName.setText("Jogador do LESTE bateu!!");
		break;
		
	    case Position.SOUTH:
		playerName.setText("Jogador do SUL bateu!!");
		break;
		
	    case Position.WEST:
		playerName.setText("Jogador do OESTE bateu!!");
		break;
		
	    case Position.NORTH:
		playerName.setText("Jogador do NORTE bateu!!");
		break;
		
	    default:
		playerName.setText("Jogador DESCONHECIDO bateu!!");
	    }

	    // setting up hand closed tiles
	    Tile theTile;
	    for (int i = 0; i < p.getHand().getClosedTiles().size(); i++)
	    {
		if (i != p.getLastTileIdx())
		{
		    theTile = (Tile)p.getHand().getClosedTiles().get(i);
		    closedTiles[i].setIcon(ImageServer.getInstance().getImage(theTile.getName() + "_0"));
		}
	    }

	    // setting up the hand opened tiles
	    ArrayList melds = new ArrayList();
	    melds.addAll(p.getHand().getOpenChi());
	    melds.addAll(p.getHand().getOpenPon());
	    melds.addAll(p.getHand().getOpenKan());
	    melds.addAll(p.getHand().getClosedKan());
	    for (int i = 0; i < melds.size(); i++) openMelds[i].notify((OpenMeld)melds.get(i));

	    // setting up the last tile
	    if (p.getLastAction() == Action.TSUMO) drawType.setText("Tsumo");
	    else drawType.setText("Ron");
	    theTile = (Tile)p.getHand().getClosedTiles().get(p.getLastTileIdx());
	    lastTile.setIcon(ImageServer.getInstance().getImage(theTile.getName() + "_0"));
	    selectedTile.setIcon(ImageServer.getInstance().getImage("selection"));
	    
	}
	else
	{
	    drawType.setText("");
	    lastTile.setIcon(null);
	    selectedTile.setIcon(null);
	    playerImage.setIcon(null);
	    playerName.setText("Empate...");
	}

	// setting up the list label
	short totalHan = (short)0;
	String listString = new String();
	listString = "<html>" + 
	                "<table border = 1>" + 
	                   "<tr>" +
                              "<td> <B> Batida </B> </td>" +
	                      "<td> <B> Han </B> </td>" +
	                   "</tr>";

	Batida b;
	for (int i = 0; i < Result.getInstance().getBatidas().size(); i++)
	{
	    b = (Batida)Result.getInstance().getBatidas().get(i);
	    totalHan += b.getHan();
	    listString += "<tr>" +
                             "<td> " + b.getName() + " </td>" + 
		             "<td> " + (b.getHan() >= Batida.LIMIT_HAND ? "LIMIT HAND!!" : Short.toString(b.getHan())) + "</td>" +
		          "</tr>";
	}

	listString +=   "</table>" + 
                     "</html>";
	finishedLabel.setText(listString);
	
	// setting up the total label
	String totalsString = new String();
	totalsString= "<html>" + 
                         "<table border = 1>" +
                            "<tr>" +
                               "<td> <B> Tipo de Ponto </B> </td>" +
                               "<td> <B> Valor </B> </td>" +
                            "</tr>" +
 
                            "<tr>" +
                               "<td> Han </td>" +
                               "<td> " + (totalHan >= Batida.LIMIT_HAND ? "LIMIT HAND!!" : Short.toString(totalHan)) + " </td>" +
                            "</tr>" +
 
                            "<tr>" +
                               "<td> Tali </td>" +
                               "<td> " + tali + " </td>" +
	                    "</tr>" +

                            "<tr>" +
                               "<td> <B> Montante </B> </td>" +
                               "<td> <B> " + total + " </B> </td>" +
                            "</tr>" +
                           
                         "</table>" + 
	              "</html>";
	totalsLabel.setText(totalsString);

	// setting up the score label
	String scoreString = new String();
	scoreString = "<html>" +
                         "<table border = 1>" + 
                            "<tr>" +
                               "<td> <B> Jogador </B> </td>" + 
                               "<td> <B> Pontos </B> </td>" +
                            "</tr>" +
	    
	                    "<tr>" +
	                       "<td> Leste </td>" +
                               "<td> " + score[0] + " </td>" +
                            "</tr>" +

	                    "<tr>" +
	                       "<td> Sul </td>" +
                               "<td> " + score[1] + " </td>" +
                            "</tr>" +
   
	                    "<tr>" + 
	                       "<td> Oeste </td>" +
                               "<td> " + score[2] + " </td>" +
                            "</tr>" +	
   
	                    "<tr>" +
	                       "<td> Norte </td>" +
                               "<td> " + score[3] + " </td>" +
                            "</tr>" +    
                         "</table>" +
	              "</html>";
	scoreLabel.setText(scoreString);

	// show the dialog
	this.setResizable(true);
	this.pack();
	Dimension thisDimension = this.getSize();
	this.setLocation((int)(1024 - thisDimension.getWidth())/2, (int)(768 - thisDimension.getHeight())/2);
	this.setResizable(false);
	this.show();
    }

    // private data
    private JLabel lastTile;
    private JLabel selectedTile;
    private JLabel drawType;
    private JLabel playerImage;
    private JLabel playerName;
    private JLabel[] closedTiles;
    private EastMeldPanel[] openMelds;
    private JLabel finishedLabel;
    private JLabel totalsLabel;
    private JLabel scoreLabel;
    private JButton closeButton;
}
