package mahjong;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class GameInterface extends JFrame
{
    public GameInterface()
    {
	super("Extreme Mahjong: by Bruno Pera & Dairton L. B. Filho");

	//	eastPanel.setBorder(BorderFactory.createTitledBorder(""));
	//	southPanel.setBorder(BorderFactory.createTitledBorder(""));
	//	westPanel.setBorder(BorderFactory.createTitledBorder(""));
	//	northPanel.setBorder(BorderFactory.createTitledBorder(""));
	

	this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	this.setBackground(new Color(0.20f, 0.70f, 0.20f));
	this.getContentPane().setBackground(new Color(0.20f, 0.70f, 0.20f));
        this.getContentPane().setLayout(new MyBorderLayout(5, 5));
	this.getContentPane().add(eastPanel, BorderLayout.SOUTH);
	this.getContentPane().add(southPanel, BorderLayout.EAST);
	this.getContentPane().add(westPanel, BorderLayout.NORTH);
	this.getContentPane().add(northPanel, BorderLayout.WEST);
        JPanel verticalPanel = new JPanel();
        verticalPanel.setLayout(new BoxLayout(verticalPanel, BoxLayout.Y_AXIS));
        verticalPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
        verticalPanel.add(Box.createVerticalGlue());
        JPanel horizontalPanel = new JPanel();
        horizontalPanel.setLayout(new BoxLayout(horizontalPanel, BoxLayout.X_AXIS));
        horizontalPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
        horizontalPanel.add(Box.createHorizontalGlue());
        horizontalPanel.add(centerPanel);
        horizontalPanel.add(Box.createHorizontalGlue());
        verticalPanel.add(horizontalPanel);
        verticalPanel.add(Box.createVerticalGlue());
        this.getContentPane().add(verticalPanel, BorderLayout.CENTER);
    }
    
    public void notifyInit(short dealer, short round, Hand[] hands)
    {
	centerPanel.notifyInit(dealer, round);
	eastPanel.notifyInit(hands[0]);
	southPanel.notifyInit(hands[1]);
	westPanel.notifyInit(hands[2]);
	northPanel.notifyInit(hands[3]);

	switch (dealer)
	{
	case Position.EAST:
	    eastPanel.notifyDealer();
	    break;
	case Position.SOUTH:
	    southPanel.notifyDealer();
	    break;
	case Position.WEST:
	    westPanel.notifyDealer();
	    break;
	case Position.NORTH:
	    northPanel.notifyDealer();
	    break;
	}
    }

    public void notifyDora(Tile tile)
    {
	centerPanel.notifyDora(tile);
    }

    public void notifyPlayerChanged(short player, Hand h)
    {
	switch (player)
	{
	case Position.EAST:
	    eastPanel.notifyChange(h);
	    break;
	case Position.SOUTH:
	    southPanel.notifyChange(h);
	    break;
	case Position.WEST:
	    westPanel.notifyChange(h);
	    break;
	case Position.NORTH:
	    northPanel.notifyChange(h);
	    break;
	}
    }

    public void notifyReach(short player, short tileIdx)
    {
	switch(player)
	{
	case Position.EAST:
	    eastPanel.notifyReach(tileIdx);
	    break;
	case Position.SOUTH:
	    southPanel.notifyReach(tileIdx);
	    break;
	case Position.WEST:
	    westPanel.notifyReach(tileIdx);
	    break;
	case Position.NORTH:
	    northPanel.notifyReach(tileIdx);
	    break;
	}
    }

    public void notifyRemainingTiles(short nTiles)
    {
	centerPanel.notifyRemainingTiles(nTiles);
    }

    public void notifyRealPlayerPlaying()
    {
	eastPanel.notifyRealPlayerPlaying();
    }

    public void notifySelectedTile(short tileIdx)
    {
	eastPanel.notifySelectedTile(tileIdx);
    }

    public void notifyWhoIsPlaying(short position)
    {
	centerPanel.notifyWhoIsPlaying(position);
    }

    public int invokeDiscardDialog(int actionMask, Tile tile, short stolenFrom)
    {
	DiscardDialog dDialog = new DiscardDialog(this);
	return (dDialog.show(actionMask, tile, stolenFrom));
    }

    public int invokeDrawDialog(int actionMask, Tile tile, short stolenFrom)
    {
	DrawDialog dDialog = new DrawDialog(this);
	return (dDialog.show(actionMask, tile, stolenFrom));	
    }
    
    public void invokeFinishedHandDialog(Player p, int[] score, int tali, int total)
    {
	FinishedHandDialog dialog = new FinishedHandDialog(this);
	dialog.show(p, score, tali, total);
    }

    public int getLastAction()
    {
	return (eastPanel.getLastAction());
    }

    public int getSelectedTileIdx()
    {
	return (eastPanel.getSelectedTileIdx());
    }

    private CenterPanel centerPanel = new CenterPanel();
    private EastPlayerPanel eastPanel = new EastPlayerPanel();
    private SouthPlayerPanel southPanel = new SouthPlayerPanel();
    private WestPlayerPanel westPanel = new WestPlayerPanel();
    private NorthPlayerPanel northPanel = new NorthPlayerPanel();
}






