package mahjong;

import java.util.*;

public class Hand
{
    public Hand()
    {
        discardedTiles.ensureCapacity(NUM_MAX_DISCARDED_TILES);
        closedTiles.ensureCapacity(NUM_MAX_CLOSED_TILES);
        openChi.ensureCapacity(NUM_MAX_MELDS);
        openPon.ensureCapacity(NUM_MAX_MELDS);
        openKan.ensureCapacity(NUM_MAX_MELDS);
        closedKan.ensureCapacity(NUM_MAX_MELDS);
    }

    public void addClosedTile(Tile tile)
    {
	int i;

	// search the correct position to add
	for (i = 0; i < closedTiles.size(); i++)
	{
	    Tile t = (Tile)closedTiles.get(i);
	    if (tile.getType() < t.getType()) break;
	    else if (tile.getType() == t.getType()){ if (tile.getOrder() <= t.getOrder()) break; }
	}
	closedTiles.add(i, tile);
    }

    public void removeTile(int tileIdx)
    {
	closedTiles.remove(tileIdx);
    }

    public void removeTile(Tile theTile)
    {
	closedTiles.remove(theTile);
    }

    public void addOpenMeld(short[] tilesIdx, short type, short stolenPlayer, Tile stolenTile)
    {
	Tile[] tiles = new Tile[tilesIdx.length];

	// selects the correct tiles
	for (int i = 0; i < tiles.length; i++) tiles[i] = (Tile)closedTiles.get(tilesIdx[i]);

	// create the correct meld
	switch(type)
	{
	case MeldType.CHI:
	    openChi.add(new OpenMeld(tiles, type, stolenPlayer, stolenTile));
	    break;
	case MeldType.PON:
	    openPon.add(new OpenMeld(tiles, type, stolenPlayer, stolenTile));
	    break;
	case MeldType.OPEN_KAN:
	    openKan.add(new OpenMeld(tiles, type, stolenPlayer, stolenTile));
	    break;
	case MeldType.CLOSED_KAN:
	    closedKan.add(new OpenMeld(tiles, type, stolenPlayer, stolenTile));
	    break;
	}

	// remove meld tiles from closed tiles list
	for (int i = 0; i < tiles.length; i++)
	    for (int j = 0; j < closedTiles.size(); j++)
		if (tiles[i].equals((Tile)closedTiles.get(j)))
		{
		    closedTiles.remove(j);
		    break;
		}	
    }

    public void addLateKan(short openPonIdx, Tile newTile)
    {
        // get correct pon
        OpenMeld meld = (OpenMeld)(openPon.get(openPonIdx));
        openPon.remove(openPonIdx);

        // get tiles from pon
        Tile[] ponTiles = meld.getTiles();
        Tile[] kanTiles = new Tile[ponTiles.length+1];

        // copies tiles from pon to kan
        int i;
        for (i = 0; i < ponTiles.length; i++)
            kanTiles[i] = ponTiles[i];

        // add extra tile
        kanTiles[i] = newTile;

        // finally create the late kan
        openKan.add(new OpenMeld(kanTiles, MeldType.LATE_KAN, meld.stolenFrom(), meld.getStolenTile()));                 
    }

    public void discardTile(short tileIdx)
    {
	discardedTiles.add((Tile) closedTiles.get(tileIdx));
	closedTiles.remove(tileIdx);
    }
    
    public void discardTile(Tile aTile)       // sobrecarga do mtodo
    {
	closedTiles.remove(aTile);
	discardedTiles.add(aTile);
    }

    public Tile lastDiscard()
    {
	int maxIndex;

	maxIndex = discardedTiles.size()-1;
	if (maxIndex >= 0)
	    return (Tile) discardedTiles.get(maxIndex);
	else
	    return (null);
    }
    
    public void removeDiscard()
    {
	if (discardedTiles.size() > 0)
	    discardedTiles.remove(discardedTiles.size()-1);
	else
	    System.out.println("Error removing tile from discard...");
    }

    public short getClosedTileIdx(Tile tile)
    {
	for (short i = 0; i < closedTiles.size(); i++)
	    if (closedTiles.get(i).equals(tile))
		return (i);

	return (-1);
    }

    public short getDiscardedTileIdx(Tile tile)
    {
	for (short i = 0; i < discardedTiles.size(); i++)
	    if (discardedTiles.get(i).equals(tile))
		return (i);

	return (-1);
    }

    public boolean isClosed()
    {
	// cannot have open meld to be closed
	return (openChi.size() == 0 &&
		openPon.size() == 0 &&
		openKan.size() == 0);
    }
    
    public boolean hasOpenChi()
    {
	return (openChi.size() > 0);
    }

    public boolean hasOpenPon()
    {
	return (openPon.size() > 0);
    }

    public boolean hasOpenKan()
    {
	return (openKan.size() > 0);
    }

    public boolean hasClosedKan()
    {
	return (closedKan.size() > 0);
    }

    public boolean isProperlyConfigured()
    {
	boolean[] used = new boolean[closedTiles.size()];
	Tile[] tiles = new Tile[closedTiles.size()];
	
	for (int i = 0; i < tiles.length; i++) 
	{
	    used[i] = false;
	    tiles[i] = (Tile)closedTiles.get(i);
	}
	return (recursiveCheck(tiles, used, false, true, true, null));
    }

    public boolean isProperlyConfigured(boolean runs, boolean sets, GroupChecker checker)
    {
	boolean[] used = new boolean[closedTiles.size()];
	Tile[] tiles = new Tile[closedTiles.size()];

	for (int i = 0; i < tiles.length; i++) 
	{
	    used[i] = false;
	    tiles[i] = (Tile)closedTiles.get(i);
	}
	return (recursiveCheck(tiles, used, false, runs, sets, checker));
    }
    
    private boolean recursiveCheck(Tile[] tiles, boolean[] used, boolean hasHead, 
				   boolean runs, boolean sets, GroupChecker checker)
    {
	// checking if has used all tiles
	boolean isOver = true;
	for (int i = 0; i < tiles.length; i++)
	    if (used[i] == false)
            {
		isOver = false;
		break;
	    }
	if (isOver) return (true);
	
	int[] gottenTiles = new int[3];
	if (runs)
	{
	    // trying to make a run (chi) and continue
	    if (getRun(tiles, used, gottenTiles, checker))
	    {
		if (recursiveCheck(tiles, used, hasHead, runs, sets, checker))
		    return (true);
		
		for (int i = 0; i < 3; i++)
		    used[gottenTiles[i]] = false;
	    }
	}

	if (sets)
	{
	    // trying to make a set (pon) and continue
	    if (getSet(tiles, used, gottenTiles, checker))
	    {
		if (recursiveCheck(tiles, used, hasHead, runs, sets, checker))
		    return (true);
		
		for (int i = 0; i < 3; i++)
		    used[gottenTiles[i]] = false;
	    }
	}

	// if don't have a head yet...
	if (!hasHead)
	{
	    // trying to make a head and continue
	    if (getHead(tiles, used, gottenTiles, checker))
	    {
		hasHead = true;
		if (recursiveCheck(tiles, used, hasHead, runs, sets, checker))
		    return (true);

		for (int i = 0; i < 2; i++)
		    used[gottenTiles[i]] = false;
	    }
	}
	return (false);
    }
    
    private boolean getHead(Tile[] tiles, boolean[] used, int[] gottenTiles, GroupChecker checker)
    {
	int j;

	// search for the firt non-used tile
	for (j = 0; j < used.length; j++)
	    if (used[j] == false) break;
	if (j == used.length) return (false);
	gottenTiles[0] = j;

	// search for the second non-used tile in the head (pair)
	for (++j; j < used.length; j++)
	    if (used[j] == false)
            {
		// the next non-used tile must be the same
		if (!tiles[j].equals(tiles[gottenTiles[0]]))
		    return (false);

		break;
	    }
	if (j == used.length) return (false);
	gottenTiles[1] = j;

	if (checker == null || checker.checkHead(this, gottenTiles))
        {
	    // make gotten tiles used
	    for (int i = 0; i < 2; i++) used[gottenTiles[i]] = true;
	    return (true);
	}

	return (false);
    }

    private boolean getSet(Tile[] tiles, boolean[] used, int[] gottenTiles, GroupChecker checker)
    {
	int j;

	// search for the firt non-used tile
	for (j = 0; j < used.length; j++)
	    if (used[j] == false) break;
	if (j == used.length) return (false);
	gottenTiles[0] = j;

	// search for the second non-used tile in the head (pair)
	for (++j; j < used.length; j++)
	    if (used[j] == false)
            {
		// the next non-used tile must be the same
		if (!tiles[j].equals(tiles[gottenTiles[0]]))
		    return (false);

		break;
	    }
	if (j == used.length) return (false);
	gottenTiles[1] = j;

	// search for the third non-used tile in the head (pair)
	for (++j; j < used.length; j++)
	    if (used[j] == false)
            {
		// the next non-used tile must be the same
		if (!tiles[j].equals(tiles[gottenTiles[1]]))
		    return (false);

		break;
	    }
	if (j == used.length) return (false);
	gottenTiles[2] = j;

	if (checker == null || checker.checkSet(this, gottenTiles))
	{
	    // make gotten tiles used
	    for (int i = 0; i < 3; i++) used[gottenTiles[i]] = true;
	    return (true);
	}

	return (false);
    }

    private boolean getRun(Tile[] tiles, boolean[] used, int[] gottenTiles, GroupChecker checker)
    {
	int j;

	// search for the first non-used tile
	for (j = 0; j < used.length; j++)
	    if (used[j] == false) break;
	if (j == used.length) return(false);
	gottenTiles[0] = j;

	// search for the second non-used tile in the run (chi)
	for (++j; j < used.length; j++)
	    if (used[j] == false)
	    {
		// if tile is honor or is of different type can't be made into a run (chi)
		if ((tiles[j].isHonor()) || (tiles[j].getType() != tiles[gottenTiles[0]].getType()))
		    return (false);

		// if tiles[j] is not the next, can't be made into a run (chi)
		if ((tiles[j].getOrder()-1 > tiles[gottenTiles[0]].getOrder()))
		    return (false);
		
		// if tiles[j] is the next then make it into a run (chi)
		if ((tiles[j].getOrder()-1 == tiles[gottenTiles[0]].getOrder()))
		    break;

		// if tiles are the same look the next: continue
	    }
	if (j == used.length) return (false);
	gottenTiles[1] = j;

	// search for the third non-used tile in the run (chi)
	for (++j; j < used.length; j++)
	    if (used[j] == false)
	    {
		// if tile is honor or is of different type can't be made into a run (chi)
		if ((tiles[j].isHonor()) || (tiles[j].getType() != tiles[gottenTiles[1]].getType()))
		    return (false);

		// if tiles[j] is not the next, can't be made into a run (chi)
		if ((tiles[j].getOrder()-1 > tiles[gottenTiles[1]].getOrder()))
		    return (false);
		
		// if tiles[j] is the next then make it into a run (chi)
		if ((tiles[j].getOrder()-1 == tiles[gottenTiles[1]].getOrder()))
		    break;

		// if tiles are the same look the next: continue
	    }
	if (j == used.length) return (false);
	gottenTiles[2] = j;


	if (checker == null || checker.checkRun(this, gottenTiles))
	{
	    // make gotten tiles used
	    for (int i = 0; i < 3; i++) used[gottenTiles[i]] = true;
	    return (true);
	}
	
	return (false);
    }

    public void clear()
    {
	closedTiles.removeAll(closedTiles);
	discardedTiles.removeAll(discardedTiles);
	openChi.removeAll(openChi);
	openPon.removeAll(openPon);
	openKan.removeAll(openKan);
	closedKan.removeAll(closedKan);
    }
    
    public ArrayList getClosedTiles()
    {
	return (closedTiles);
    }

    public ArrayList getDiscardedTiles()
    {
	return (discardedTiles);
    }

    public ArrayList getOpenChi()
    {
	return (openChi);
    }

    public ArrayList getOpenPon()
    {
	return (openPon);
    }

    public ArrayList getOpenKan()
    {
	return (openKan);
    }

    public ArrayList getClosedKan()
    {
	return (closedKan);
    }

    public void resetPoints()
    {
	for (int i = 0; i < closedTiles.size(); i++)
	    ((Tile)closedTiles.get(i)).initPoints();
    }
    
    public int getBiggerScore()
    {
	Tile maxScore;
	
	maxScore = (Tile) closedTiles.get(0);

	for (int i = 1; i < closedTiles.size(); i++)
	    if (((Tile) closedTiles.get(i)).getPoints() >= maxScore.getPoints())
		maxScore = (Tile) closedTiles.get(i);
	return (this.getClosedTileIdx(maxScore));
    }
    
    public void printTest()
    {
	System.out.println("closedTiles....: " + closedTiles.size());
	for (int i = 0; i < closedTiles.size(); i++) ((Tile)closedTiles.get(i)).printTest();

	System.out.println("discardedTiles.: " + discardedTiles.size());
	for (int i = 0; i < discardedTiles.size(); i++) ((Tile)discardedTiles.get(i)).printTest();
	
	System.out.println("openChi........: " + openChi.size());
	for (int i = 0; i < openChi.size(); i++) ((OpenMeld)openChi.get(i)).printTest();

	System.out.println("openPon........: " + openPon.size());
	for (int i = 0; i < openPon.size(); i++) ((OpenMeld)openPon.get(i)).printTest();

	System.out.println("openKan........: " + openKan.size());
	for (int i = 0; i < openKan.size(); i++) ((OpenMeld)openKan.get(i)).printTest();

        System.out.println("closedKan......: " + closedKan.size());
        for (int i = 0; i < closedKan.size(); i++) ((OpenMeld)closedKan.get(i)).printTest();
    }

    // private data
    private ArrayList discardedTiles = new ArrayList();
    private ArrayList closedTiles = new ArrayList();
    private ArrayList openChi = new ArrayList();
    private ArrayList openPon = new ArrayList();
    private ArrayList openKan = new ArrayList();
    private ArrayList closedKan = new ArrayList();

    // some definitions
    public static final short NUM_MAX_CLOSED_TILES = 14;
    public static final short NUM_MAX_DISCARDED_TILES = 20;
    public static final short NUM_MAX_MELDS = 4;
}






   


