package mahjong;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;

public class ImageServer
{
    public static ImageServer getInstance()
    {
	return (imageServer);
    }

    public ImageIcon getImage(Object key)
    {
	ImageIcon result = ((ImageIcon)imageMap.get(imagePath + key + ".gif"));
	if (result == null) System.out.println("Image " + key + " not found...");
	return result;
    }
    
    private ImageServer()
    {
	loadTileImages();
	loadOtherImages();
    }

    private void loadTileImages()
    {
	int i;

	// Loading PINZU images
	System.out.println("Loading PINZU images...");
	for (i = 1; i < 10; i++)
	{
            for (int j = 0; j < 360; j += 90)
            {
                String imageName = imagePath + Tile.getName(TileType.PINZU, (short)i) + "_" + j + ".gif";
                System.out.println("...Loading image " + imageName + "...");
                URL imageURL  = ClassLoader.getSystemResource(imageName);
                ImageIcon imageIcon = new ImageIcon(imageURL);
                imageMap.put(imageName, imageIcon);
            }
	}

	// Loading SOUZU images
	System.out.println("Loading SOUNZU images...");
	for (i = 1; i <= 9; i++)
	{
            for (int j = 0; j < 360; j += 90)
            {
                String imageName = imagePath + Tile.getName(TileType.SOUZU, (short)i) + "_" + j + ".gif";
                System.out.println("...Loading image " + imageName + "...");
                URL imageURL  = ClassLoader.getSystemResource(imageName);
                ImageIcon imageIcon = new ImageIcon(imageURL);
                imageMap.put(imageName, imageIcon);
            }
	}

	// Loading WANZU images
	System.out.println("Loading WANZU images...");
	for (i = 1; i <= 9; i++)
	{
            for (int j = 0; j < 360; j += 90)
            {
                String imageName = imagePath + Tile.getName(TileType.WANZU, (short)i) + "_" + j + ".gif";
                System.out.println("...Loading image " + imageName + "...");
                URL imageURL  = ClassLoader.getSystemResource(imageName);
                ImageIcon imageIcon = new ImageIcon(imageURL);
                imageMap.put(imageName, imageIcon);
            }
        }

	// Loading KAZEHAI images
	System.out.println("Loading KAZEHAI images...");
	for (i = 1; i <= 4; i++)
	{
            for (int j = 0; j < 360; j += 90)
            {
                String imageName = imagePath + Tile.getName(TileType.KAZEHAI, (short)i) + "_" + j + ".gif";
                System.out.println("...Loading image " + imageName + "...");
                URL imageURL  = ClassLoader.getSystemResource(imageName);
                ImageIcon imageIcon = new ImageIcon(imageURL);
                imageMap.put(imageName, imageIcon);
            }
	}

	// Loading SANGENPAI images
	System.out.println("Loading SANGENPAI images...");
	for (i = 1; i <= 3; i++)
	{
            for (int j = 0; j < 360; j += 90)
            {
                String imageName = imagePath + Tile.getName(TileType.SANGENPAI, (short)i) + "_" + j + ".gif";
                System.out.println("...Loading image " + imageName + "...");
                URL imageURL  = ClassLoader.getSystemResource(imageName);
                ImageIcon imageIcon = new ImageIcon(imageURL);
                imageMap.put(imageName, imageIcon);
            }
        }

        // Loading backside tile images
        System.out.println("Loading BACKSIDE image");
        {
           for (int j = 0; j < 360; j += 90)
           {
               String imageName = imagePath + "back_" + j + ".gif";
               System.out.println("...Loading image " + imageName + "...");
               URL imageURL = ClassLoader.getSystemResource(imageName);
               ImageIcon imageIcon = new ImageIcon(imageURL);
               imageMap.put(imageName, imageIcon);
            }
        }
    }
    
    private void loadOtherImages()
    {
	String imageName;
	URL imageURL;

        // Loading 1000 points chip images
        System.out.println("Loading CHIP images");
        for (int j = 0; j < 360; j += 90)
        {
            imageName = imagePath + "chip_1000_" + j + ".gif";
            System.out.println("...Loading image " + imageName + "...");
            imageURL = ClassLoader.getSystemResource(imageName);
            imageMap.put(imageName, new ImageIcon(imageURL));
        }

        // Loading dice images
        System.out.println("Loading DICE images");
        for (int j = 0; j < 360; j += 90)
        {
            imageName = imagePath + "dice_" + j + ".gif";
            System.out.println("...Loading image " + imageName + "...");
            imageURL = ClassLoader.getSystemResource(imageName);
            imageMap.put(imageName, new ImageIcon(imageURL));
        }

	// Loading the Tile Selection image
	System.out.println("Loading SELECTION image");
	imageName = imagePath + "selection.gif";
	System.out.println("...Loading image " + imageName + "...");
	imageURL = ClassLoader.getSystemResource(imageName);
	imageMap.put(imageName, new ImageIcon(imageURL));

	// Loading the East Round image
	System.out.println("Loading EAST_ROUND image");
	imageName = imagePath + "east_round.gif";
	System.out.println("...Loading image " + imageName + "...");
	imageURL = ClassLoader.getSystemResource(imageName);
	imageMap.put(imageName, new ImageIcon(imageURL));

	// Loading the South Round image
	System.out.println("Loading SOUTH_ROUND image");
	imageName = imagePath + "south_round.gif";
	System.out.println("...Loading image " + imageName + "...");
	imageURL = ClassLoader.getSystemResource(imageName);
	imageMap.put(imageName, new ImageIcon(imageURL));

	// Loading the Arrow images
	System.out.println("Loading ARROW images");
	for (int j = 0; j < 360; j += 90)
	{
	    imageName = imagePath + "arrow_" + j + ".gif";
	    System.out.println("...Loading image " + imageName + "...");
	    imageURL = ClassLoader.getSystemResource(imageName);
	    imageMap.put(imageName, new ImageIcon(imageURL));
	}

	// Loading Player Images
	System.out.println("Loading PLAYER images");
	for (int j = 90; j < 360; j+= 90)
	{
	    imageName = imagePath + "player_" + j + ".gif";
	    System.out.println("...Loading image " + imageName + "...");
	    imageURL = ClassLoader.getSystemResource(imageName);
	    imageMap.put(imageName, new ImageIcon(imageURL));	    	    	    	    
	}
    }

    private String imagePath = "mahjong/images/";
    private HashMap imageMap = new HashMap(151);
    private static ImageServer imageServer = new ImageServer(); // singleton
}






