package mahjong;

import java.util.*;

public abstract class Player
{
    public Player(short position)
    {
	this.position = position;
	this.reach = false;
	this.lastTileIdx = -1;
	this.lastAction = Action.NONE;
	this.hand = new Hand();
	this.analyser = new Analyser(this);
	this.score = 0;
    }

    public boolean isTenpai()
    {
	// testing PINZU tiles
	Tile theTile;
	for (int i = 1; i <= 9; i++)
	{
	    theTile = new Tile(TileType.PINZU, (short)i);
	    hand.addClosedTile(theTile);
	    if (hand.isProperlyConfigured())
            {
		hand.removeTile(theTile);
		return(true);
	    }
	    hand.removeTile(theTile);
	}

	// testing SOUZU tiles
	for (int i = 1; i <= 9; i++)
	{
	    theTile = new Tile(TileType.SOUZU, (short)i);
	    hand.addClosedTile(theTile);
	    if (hand.isProperlyConfigured())
            {
		hand.removeTile(theTile);
		return(true);
	    }
	    hand.removeTile(theTile);
	}

	// testing WANZU tiles
	for (int i = 1; i <= 9; i++)
	{
	    theTile = new Tile(TileType.WANZU, (short)i);
	    hand.addClosedTile(theTile);
	    if (hand.isProperlyConfigured())
            {
		hand.removeTile(theTile);
		return(true);
	    }
	    hand.removeTile(theTile);
	}

	// testing KAZEHAI tiles
	for (int i = 1; i <= 4; i++)
	{
	    theTile = new Tile(TileType.KAZEHAI, (short)i);
	    hand.addClosedTile(theTile);
	    if (hand.isProperlyConfigured())
            {
		hand.removeTile(theTile);
		return(true);
	    }
	    hand.removeTile(theTile);
	}

	// testing SANGENPAI tiles
	for (int i = 1; i <= 9; i++)
	{
	    theTile = new Tile(TileType.SANGENPAI, (short)i);
	    hand.addClosedTile(theTile);
	    if (hand.isProperlyConfigured())
            {
		hand.removeTile(theTile);
		return(true);
	    }
	    hand.removeTile(theTile);
	}

	return (false);
    }

    public int getLastTileIdx()
    {
	return (lastTileIdx);
    }

    public void reset()
    {
	reach = false;
	lastTileIdx = -1;
	lastAction = Action.NONE;
	hand.clear();
    }

    public void setScore(int score)
    {
	this.score = score;
    }

    public int getScore()
    {
	return (score);
    }
    
    public void setReach(boolean isReach)
    {
	reach = isReach;
    }

    public boolean isReach()
    {
	return (reach);
    }

    public short getPosition()
    {
	return (position);
    }

    public Hand getHand()
    {
	return (hand);
    }

    public void setLastAction(int action)
    {
	lastAction = action;
    }

    public int getLastAction()
    {
	return (lastAction);
    }

    protected void makeLeftChi(Tile stolenTile)
    {
	short[] tilesIdx = new short[2];

	// locate the correct tiles
	tilesIdx[0] = hand.getClosedTileIdx(stolenTile.next());
	tilesIdx[1] = hand.getClosedTileIdx(stolenTile.next().next());
	
	// create the correct meld
	hand.addOpenMeld(tilesIdx, MeldType.CHI, StolenPlayer.LEFT, stolenTile);
    }

    protected void makeCenterChi(Tile stolenTile)
    {
	short[] tilesIdx = new short[2];
	
	// locate the correct tiles
	tilesIdx[0] = hand.getClosedTileIdx(stolenTile.previous());
	tilesIdx[1] = hand.getClosedTileIdx(stolenTile.next());

	// create the correct meld
	hand.addOpenMeld(tilesIdx, MeldType.CHI, StolenPlayer.LEFT, stolenTile);
    }

    protected void makeRightChi(Tile stolenTile)
    {
	short[] tilesIdx = new short[2];
	
	// locate the correct tiles
	tilesIdx[0] = hand.getClosedTileIdx(stolenTile.previous().previous());
	tilesIdx[1] = hand.getClosedTileIdx(stolenTile.previous());
	
	// create the correct meld
	hand.addOpenMeld(tilesIdx, MeldType.CHI, StolenPlayer.LEFT, stolenTile);
    }

    protected void makePon(Tile stolenTile, short stolenPlayer)
    {
	short[] tilesIdx = new short[2];

	// locate the correct tiles
	tilesIdx[0] = hand.getClosedTileIdx(stolenTile);
	tilesIdx[1] = (short)(tilesIdx[0] + 1);
	
	// create the correct meld
	hand.addOpenMeld(tilesIdx, MeldType.PON, stolenPlayer, stolenTile);
    }

    protected void makeOpenKan(Tile stolenTile, short stolenPlayer)
    {
	short[] tilesIdx = new short[3];

	// locate the correct tiles
	tilesIdx[0] = hand.getClosedTileIdx(stolenTile);
	tilesIdx[1] = (short)(tilesIdx[0] + 1);
	tilesIdx[2] = (short)(tilesIdx[0] + 2);

	// create the correct meld
	hand.addOpenMeld(tilesIdx, MeldType.OPEN_KAN, stolenPlayer, stolenTile);
    }

    protected void makeRon(Tile stolenTile, short stolenPlayer)
    {
	// fill in the result singleton
	hand.addClosedTile(stolenTile);
	lastTileIdx = hand.getClosedTileIdx(stolenTile);
	analyser.analyse(Action.RON, true);
    }

    public boolean makeAnalyse (int action, boolean finish)
    {
	return (analyser.analyse(action, finish));   
    }
    
    public void receiveTile(Tile tile, int action, short stolenPosition)
    {
	short stolenPlayer;

	if (position == Position.nextPosition(stolenPosition))
	    stolenPlayer = StolenPlayer.LEFT;
	else if (position == Position.prevPosition(stolenPosition))
	    stolenPlayer = StolenPlayer.RIGHT;
	else if (position == Position.nextPosition(Position.nextPosition(stolenPosition)))
	    stolenPlayer = StolenPlayer.ACROSS;
	else
	    stolenPlayer = StolenPlayer.NONE;

	switch (action)
	{
	case Action.DRAW:
	    hand.addClosedTile(tile);
	    lastTileIdx = hand.getClosedTileIdx(tile);
	    break;
	case Action.LEFT_CHI:
	    makeLeftChi(tile);
	    break;
	case Action.CENTER_CHI:
	    makeCenterChi(tile);
	    break;
	case Action.RIGHT_CHI:
	    makeRightChi(tile);
	    break;
	case Action.PON:
	    makePon(tile, stolenPlayer);
	    break;
	case Action.OPEN_KAN:
	    makeOpenKan(tile, stolenPlayer);
	    break;
	case Action.RON:
	    makeRon(tile, stolenPlayer);
	    break;
	}
    }

    // public abstract methods
    public abstract Tile discard(int drawType);
    public abstract int wantTile(Tile theTile, short stolenPlayer);

    // private data
    protected boolean reach;
    protected short position;
    protected short lastTileIdx;
    protected int lastAction;
    protected int score;
    protected Hand hand;
    protected Analyser analyser;
}
