package mahjong;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class PlayerPanel extends JPanel
{
    public PlayerPanel()
    {
	super();
	this.setBackground(new Color(0.20f, 0.70f, 0.20f));
	meldsPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	discardedTilesPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	reachPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
	closedTilesPanel.setBackground(new Color(0.20f, 0.70f, 0.20f));
    }

    protected void makeInitHand()
    {
	ImageServer server = ImageServer.getInstance();

	for (int i = 0; i < closedTiles.length; i++)
	    closedTiles[i].setIcon(server.getImage("back_" + defaultRotation));

	for (int i = 0; i < discardedTiles.length; i++)
	    discardedTiles[i].setIcon(server.getImage("back_" + defaultRotation));

	diceLabel.setIcon(server.getImage("dice_" + defaultRotation));
	reachLabel.setIcon(server.getImage("chip_1000_" + defaultRotation));
    }
    
    public void notifyInit(Hand h)
    {
	updateClosedTilesPanel(h);
	updateReachPanel();
	updateDiscardedTilesPanel(h);
	updateMeldsPanel(h);
        revalidate();
    }

    public void notifyChange(Hand h)
    {
	updateClosedTilesPanel(h);
	updateDiscardedTilesPanel(h);
	updateMeldsPanel(h);
	revalidate();
    }

    protected void updateClosedTilesPanel(Hand h)
    {
	int i;
	ArrayList tiles = h.getClosedTiles();
	ImageServer imageServer = ImageServer.getInstance();

	System.out.println("" + tiles.size());
	for (i = 0; i < tiles.size(); i++)
            closedTiles[i].setIcon(imageServer.getImage("back" + "_" + defaultRotation));

	for (; i < closedTiles.length; i++)
	    closedTiles[i].setIcon(null);

    }

    protected void updateReachPanel()
    {
	diceLabel.setIcon(null);
	reachLabel.setIcon(null);
	reachIdx = -1;
    }
   
    protected void updateDiscardedTilesPanel(Hand h)
    {
	int i;
	ArrayList tiles = h.getDiscardedTiles();
	ImageServer imageServer = ImageServer.getInstance();

	for (i = 0; i < tiles.size(); i++)
	{
	    if (i != reachIdx)
		discardedTiles[i].setIcon(imageServer.getImage(((Tile)tiles.get(i)).getName() + 
							       "_" + defaultRotation));
	    else
		discardedTiles[i].setIcon(imageServer.getImage(((Tile)tiles.get(i)).getName() + 
							       "_" + (defaultRotation+90)%360));
	}

	for (; i < discardedTiles.length; i++)
	    discardedTiles[i].setIcon(null);
    }

    protected void updateMeldsPanel(Hand h)
    {
	int i;
	ArrayList openMelds = new ArrayList(4);

	openMelds.addAll(h.getClosedKan());
	openMelds.addAll(h.getOpenKan());
	openMelds.addAll(h.getOpenPon());
	openMelds.addAll(h.getOpenChi());

	for (i = 0; i < openMelds.size(); i++)
	    melds[i].notify((OpenMeld)openMelds.get(i));
	
	for (; i < melds.length; i++)
	    melds[i].notify(null);
    }

    public void notifyDealer()
    {
	diceLabel.setIcon(ImageServer.getInstance().getImage("dice_" + defaultRotation));
	revalidate();
    }

    public void notifyReach(short tileIdx)
    {
	reachLabel.setIcon(ImageServer.getInstance().getImage("chip_1000_" + defaultRotation));
	reachIdx = tileIdx;
	revalidate();
    }

    // melds panel
    protected JPanel meldsPanel = new JPanel();
    protected MeldPanel melds[] = new MeldPanel[4];
    
    // discarded tiles panel
    protected JPanel discardedTilesPanel = new JPanel();
    protected JLabel[] discardedTiles = new JLabel[20];

    // reach panel
    protected JPanel reachPanel = new JPanel();
    protected JLabel diceLabel = new JLabel();
    protected JLabel reachLabel = new JLabel();
    protected short reachIdx = -1; // index of the reach'ed tile on the discard list

    // closed tiles panel
    protected JPanel closedTilesPanel = new JPanel();
    protected TaggedLabel[] closedTiles = new TaggedLabel[14];

    protected int defaultRotation;
}

class MeldPanel extends JPanel
{
    MeldPanel()
    {
	super();
	this.setBackground(new Color(0.20f, 0.70f, 0.20f));
	for (int i = 0; i < 4; i++)
	{
	    coupleTilePanel[i] = new JPanel();
	    coupleTilePanel[i].setBackground(new Color(0.20f, 0.70f, 0.20f));
	}
    }
    
    void notify(OpenMeld m)
    {
	for (int i = 0; i < 4; i++)
	{
	    upperTiles[i].setIcon(null);
	    lowerTiles[i].setIcon(null);
	}

	if (m != null)
	{
	    ImageServer imageServer = ImageServer.getInstance();
	    Tile[] tiles = m.getTiles();
	
	    switch(m.getType())
            {
	    case MeldType.CHI:
	    case MeldType.PON:
	    case MeldType.LATE_KAN:
		switch(m.stolenFrom())
		{
		case StolenPlayer.LEFT:
		    lowerTiles[0].setIcon(imageServer.getImage(m.getStolenTile().getName() + 
							       "_" + (defaultRotation+90)%360));
		    lowerTiles[1].setIcon(imageServer.getImage(tiles[0].getName() +
							       "_" + defaultRotation));
		    lowerTiles[2].setIcon(imageServer.getImage(tiles[1].getName() + 
							       "_" + defaultRotation));
		    if (m.getType() == MeldType.LATE_KAN)
			upperTiles[0].setIcon(imageServer.getImage(tiles[2].getName() + 
								   "_" + (defaultRotation+90)%360));
		    break;

		case StolenPlayer.ACROSS:
		    lowerTiles[0].setIcon(imageServer.getImage(tiles[0].getName() + 
							       "_" + defaultRotation));
		    lowerTiles[1].setIcon(imageServer.getImage(m.getStolenTile().getName() + 
							       "_" + (defaultRotation+90)%360));
		    lowerTiles[2].setIcon(imageServer.getImage(tiles[1].getName() + 
							       "_" + defaultRotation));
		    if (m.getType() == MeldType.LATE_KAN)
			upperTiles[1].setIcon(imageServer.getImage(tiles[2].getName() + 
								   "_" + (defaultRotation+90)%360));
		    break;

		case StolenPlayer.RIGHT:
		    lowerTiles[0].setIcon(imageServer.getImage(tiles[0].getName() + 
							       "_" + defaultRotation));
		    lowerTiles[1].setIcon(imageServer.getImage(tiles[1].getName() +
							       "_" + defaultRotation));
		    lowerTiles[2].setIcon(imageServer.getImage(m.getStolenTile().getName() + 
							       "_" + (defaultRotation+90)%360));
		    if (m.getType() == MeldType.LATE_KAN)
			upperTiles[2].setIcon(imageServer.getImage(tiles[2].getName() + 
								   "_" + (defaultRotation+90)%360));
		    break;
		}
		break;

	    case MeldType.OPEN_KAN:
		switch(m.stolenFrom())
		{
		case StolenPlayer.LEFT:
		    lowerTiles[0].setIcon(imageServer.getImage(m.getStolenTile().getName() + 
							       "_" + (defaultRotation+90)%360));
		    lowerTiles[1].setIcon(imageServer.getImage(tiles[0].getName() +
							       "_" + defaultRotation));
		    lowerTiles[2].setIcon(imageServer.getImage(tiles[1].getName() + 
							       "_" + defaultRotation));    
		    lowerTiles[3].setIcon(imageServer.getImage(tiles[2].getName() + 
							       "_" + defaultRotation));    
		    break;

		case StolenPlayer.ACROSS:
		    lowerTiles[0].setIcon(imageServer.getImage(tiles[0].getName() + 
							       "_" + defaultRotation));
		    lowerTiles[1].setIcon(imageServer.getImage(m.getStolenTile().getName() + 
							       "_" + (defaultRotation+90)%360));
		    lowerTiles[2].setIcon(imageServer.getImage(tiles[1].getName() + 
							       "_" + defaultRotation));
		    lowerTiles[3].setIcon(imageServer.getImage(tiles[2].getName() + 
							       "_" + defaultRotation));
		    break;
		    
		case StolenPlayer.RIGHT:
		    lowerTiles[0].setIcon(imageServer.getImage(tiles[0].getName() + 
							       "_" + defaultRotation));
		    lowerTiles[1].setIcon(imageServer.getImage(tiles[1].getName() +
							       "_" + defaultRotation));
		    lowerTiles[2].setIcon(imageServer.getImage(tiles[2].getName() +
							       "_" + defaultRotation));
		    lowerTiles[3].setIcon(imageServer.getImage(m.getStolenTile().getName() + 
							       "_" + (defaultRotation+90)%360));  
		    break;
		}
		break;

	    case MeldType.CLOSED_KAN:
		lowerTiles[0].setIcon(imageServer.getImage("back_" + defaultRotation));
		lowerTiles[1].setIcon(imageServer.getImage(tiles[1].getName() +
							   "_" + defaultRotation));
		lowerTiles[2].setIcon(imageServer.getImage(tiles[2].getName() +
							   "_" + defaultRotation));
		lowerTiles[3].setIcon(imageServer.getImage("back_" + defaultRotation));
		break;
	    }
	}
    }

    // Meld Panel components
    protected JLabel[] upperTiles = new JLabel[4]; //used for late kan
    protected JLabel[] lowerTiles = new JLabel[4];
    protected JPanel[] coupleTilePanel = new JPanel[4];

    protected int defaultRotation;
}

class TaggedLabel extends JLabel
{   
    TaggedLabel(int tag)
    {
	super();
	this.tag = tag;
    }

    int getTag() { return (tag); }

    private int tag;
}



