package mahjong;

public class RealPlayer extends Player
{
    public RealPlayer(short position)
    {
	super(position);
    }

    public Tile discard(int drawType)
    {
	GameInterface gameInterface = Table.getInstance().getGameInterface();
	gameInterface.notifySelectedTile(lastTileIdx);

	if (drawType == Action.DRAW)
	{
	    Tile lastTile = (Tile)hand.getClosedTiles().get(lastTileIdx);
	    int actionMask = Action.NONE;

	    // can make closed kan?
	    int nSameTiles = 0 ;
	    for (int i = 0; i < hand.getClosedTiles().size(); i++)
		if (((Tile)hand.getClosedTiles().get(i)).equals(lastTile))
		    nSameTiles++;
	    if (nSameTiles >= 4) actionMask = actionMask | Action.CLOSED_KAN;

	    // can make late kan?
	    OpenMeld m;
	    Tile t;
	    short stolenFrom = Position.EAST;
	    for (int i = 0; i < hand.getOpenPon().size(); i++)
	    {
		m = (OpenMeld)hand.getOpenPon().get(i);
		t = (Tile)m.getTiles()[0];
		if (t.equals((Tile)hand.getClosedTiles().get(lastTileIdx)))
		{
		    actionMask = actionMask | Action.LATE_KAN;
		    switch (m.stolenFrom())
		    {
		    case StolenPlayer.LEFT:
			stolenFrom = Position.NORTH;
			break;
		    case StolenPlayer.ACROSS:
			stolenFrom = Position.WEST;
			break;
		    case StolenPlayer.RIGHT:
			stolenFrom = Position.SOUTH;
			break;
		    }
		    break;
		}
	    }

	    // can make tsumo?
  	    if (analyser.analyse(Action.TSUMO, false)) actionMask = actionMask | Action.TSUMO;
	    
	    if (actionMask != Action.NONE)
	    {
		int answer = gameInterface.invokeDrawDialog(actionMask, 
							    (Tile)hand.getClosedTiles().get(lastTileIdx), 
							    stolenFrom);
		if (answer == Action.LATE_KAN)
		{
		    // making a late kan
		    int i;
		    for (i = 0; i < hand.getOpenPon().size(); i++)
		    {
			if (((OpenMeld)hand.getOpenPon().get(i)).getTiles()[0].equals(hand.getClosedTiles().get(lastTileIdx)))
			 {
			     hand.addLateKan((short)i, (Tile)hand.getClosedTiles().get(lastTileIdx));
			     hand.getClosedTiles().remove(lastTileIdx);
			     lastAction = Action.LATE_KAN;
			     return (null);
			 }
		    }
		}
		
		if (answer == Action.CLOSED_KAN)
		{
		    // making a closed kan;
		    short j = hand.getClosedTileIdx((Tile)hand.getClosedTiles().get(lastTileIdx));
		    if (j + 3 <  hand.getClosedTiles().size())
		    {
			if(((Tile)hand.getClosedTiles().get(j)).equals((Tile)hand.getClosedTiles().get(j+1)) &&
			   ((Tile)hand.getClosedTiles().get(j)).equals((Tile)hand.getClosedTiles().get(j+2)) &&
			   ((Tile)hand.getClosedTiles().get(j)).equals((Tile)hand.getClosedTiles().get(j+3)))
			    {
				hand.addOpenMeld(new short[] {j, (short)(j+1), (short)(j+2), (short)(j+3)}, 
						 MeldType.CLOSED_KAN, StolenPlayer.NONE, null);
				lastAction = Action.CLOSED_KAN;
				return (null);
			    }
		    }
		}
		
		if (answer == Action.TSUMO)
		{
		    // fill int the result singleton
		    analyser.analyse(Action.TSUMO, true);
		    lastAction = Action.TSUMO;
		    return (null);
		}
	    }
	}

	// while user dont make valid move
	int tileIdx = -1;
	while(true)
	{
	    gameInterface.notifyRealPlayerPlaying();
	    while (gameInterface.getLastAction() == UserAction.USER_ACTION_NONE) nap();
	    lastAction = gameInterface.getLastAction();
	    tileIdx = gameInterface.getSelectedTileIdx();

	    if (tileIdx < 0 || tileIdx >= hand.getClosedTiles().size()) continue;

	    // just discard
	    if (lastAction == UserAction.USER_ACTION_DISCARD)
	    {
		if (reach && (tileIdx != lastTileIdx)) continue;
		hand.discardTile((short)tileIdx);
		lastAction = Action.NONE;
		return(hand.lastDiscard());
	    }

	    if (lastAction == UserAction.USER_ACTION_KAN)
	    {
		// try making a late kan
		int i;
		for (i = 0; i < hand.getOpenPon().size(); i++)
		{
		    if (((OpenMeld)hand.getOpenPon().get(i)).getTiles()[0].equals(hand.getClosedTiles().get(tileIdx)))
		    {
			hand.addLateKan((short)i, (Tile)hand.getClosedTiles().get(tileIdx));
			hand.getClosedTiles().remove(tileIdx);
			lastAction = Action.LATE_KAN;
			return (null);
		    }
		}

		// try making a closed kan;
		short j = hand.getClosedTileIdx((Tile)hand.getClosedTiles().get(tileIdx));
		if (j + 3 <  hand.getClosedTiles().size())
		{
		    if ( ((Tile)hand.getClosedTiles().get(j)).equals((Tile)hand.getClosedTiles().get(j+1)) &&
			 ((Tile)hand.getClosedTiles().get(j)).equals((Tile)hand.getClosedTiles().get(j+2)) &&
			 ((Tile)hand.getClosedTiles().get(j)).equals((Tile)hand.getClosedTiles().get(j+3)))
		    {
			hand.addOpenMeld(new short[] {j, (short)(j+1), (short)(j+2), (short)(j+3)}, 
					 MeldType.CLOSED_KAN, StolenPlayer.NONE, null);
			lastAction = Action.CLOSED_KAN;
			return (null);
		    }
		}
	    }
	    
	    // discard and set last action as REACH
	    if (lastAction == UserAction.USER_ACTION_REACH)
	    {
		if (!reach && hand.isClosed())
		{
		    hand.discardTile((short)tileIdx);
		    reach = true;
		    lastAction = Action.REACH;		    
		    return(hand.lastDiscard());		
		}
	    }
	}
    }

    public int wantTile(Tile theTile, short stolenPlayer)
    {
	int actionMask = Action.NONE;
	short stolenFrom;

	switch (stolenPlayer)
	{
	case StolenPlayer.LEFT:
	    stolenFrom = Position.NORTH;
	    break;
	case StolenPlayer.ACROSS:
	    stolenFrom = Position.WEST;
	    break;
	case StolenPlayer.RIGHT:
	    stolenFrom = Position.SOUTH;
	    break;
	default:
	    stolenFrom = Position.EAST;
	    break;
	}

	if (stolenPlayer == StolenPlayer.LEFT && !theTile.isHonor() && !reach)
	{
	    Tile prevTile = theTile.previous();
	    Tile nextTile = theTile.next();

	    // can make left chi?
	    if (theTile.getOrder() <= 7 &&
		hand.getClosedTileIdx(nextTile) >= 0 &&
		hand.getClosedTileIdx(nextTile.next()) >= 0)
		actionMask = actionMask | Action.LEFT_CHI;

	    // can make center chi?
	    if (theTile.getOrder() <= 8 &&
		theTile.getOrder() >= 2 &&
		hand.getClosedTileIdx(prevTile) >= 0 &&
		hand.getClosedTileIdx(nextTile) >= 0)
		actionMask = actionMask | Action.CENTER_CHI;

	    // can make right chi?
	    if (theTile.getOrder() >= 3 &&
		hand.getClosedTileIdx(prevTile) >= 0 &&
		hand.getClosedTileIdx(prevTile.previous()) >= 0)
		actionMask = actionMask | Action.RIGHT_CHI;
	}
	
	int nSameTiles = 0;
	for (int i = 0; i < hand.getClosedTiles().size(); i++)
	    if (((Tile)hand.getClosedTiles().get(i)).equals(theTile))
		nSameTiles++;

	//can make pon?
	if (nSameTiles >= 2 && !reach) actionMask = actionMask | Action.PON;
	
	// can make open kan?
	if (nSameTiles >= 3 && !reach) actionMask = actionMask | Action.OPEN_KAN;

	// can ron?
	hand.addClosedTile(theTile);
	if (analyser.analyse(Action.RON, false)) actionMask = actionMask | Action.RON;
	hand.removeTile(theTile);
	
	if (actionMask != Action.NONE)
	    return (Table.getInstance().getGameInterface().invokeDiscardDialog(actionMask, theTile, stolenFrom));       
	else
	    return (Action.NONE);
    }

    private void nap()
    {
	try { Thread.sleep(NAP_TIME); }
	catch (InterruptedException e) { System.out.println(e); }
    }

    private static final int NAP_TIME = 100;
}
