package mahjong;

import java.util.*;

public class Senior extends Intelligence
{
    public Senior (Player player)
    {
	super (player);
    }


    public Tile discard (int drawType)
    {
	int tileIdx, lastTileIdx;
	
	if (drawType == Action.DRAW)
	{
	    if (player.makeAnalyse(Action.TSUMO, true))
	    {
		player.setLastAction(Action.TSUMO);
		return (null);
	    }
	    
	    int nSameTiles = 0, idx = 0, i;
	    Tile lastTile;	
	    
	    lastTileIdx = player.getLastTileIdx();
	    lastTile = (Tile) hand.getClosedTiles().get(lastTileIdx);
	    
	    for (i = 0;  i < hand.getClosedTiles().size(); i++)
		if (((Tile)hand.getClosedTiles().get(i)).equals(lastTile))
		{
		    nSameTiles++;
		    idx = i;
		}
	    if (nSameTiles == 4)
	    {
		hand.addOpenMeld(new short[] {(short)(idx-3), (short)(idx-2), (short)(idx-1), (short)idx}, 
				 MeldType.CLOSED_KAN, StolenPlayer.NONE, null);
		player.setLastAction(Action.CLOSED_KAN);
		return null;
	    }
	    
	    for (i = 0; i < hand.getOpenPon().size(); i++)
	    {
		if (((OpenMeld)hand.getOpenPon().get(i)).getTiles()[0].equals(lastTile))
		{
		    hand.addLateKan((short)i, lastTile);
		    hand.getClosedTiles().remove(lastTileIdx);
		    player.setLastAction(Action.LATE_KAN);
		    return (null);
		}
	    }
	}
	
	if (this.makeReach())
	    player.setLastAction(Action.REACH);
	else
	    player.setLastAction(Action.NONE);
	/* escolhe uma peca para descartar */ 
	this.testSets();
	this.testRuns();
	tileIdx = hand.getBiggerScore();
	hand.resetPoints();
	hand.discardTile((short)tileIdx);
	return(hand.lastDiscard());
    }
    
    public int wantTile(Tile aTile, short stolenPlayer)
    {
	boolean wantRon;
	
	hand.addClosedTile(aTile);
	wantRon = player.makeAnalyse(Action.RON, false);
	hand.removeTile(aTile);
	if (wantRon)
	    return Action.RON;
	
	if (player.isReach())
	    return Action.NONE;
	else
	{
	    int nSameTiles = 0;
	    for (int i = 0;  i < hand.getClosedTiles().size(); i++)
		if (((Tile)hand.getClosedTiles().get(i)).equals(aTile))
		    nSameTiles++;
	    if (nSameTiles == 3)
		return Action.OPEN_KAN;
	    if (nSameTiles == 2)
		return Action.PON;
	    if (!aTile.isHonor() && (stolenPlayer == StolenPlayer.LEFT))
	    {
		Tile prevTile = aTile.previous();
		Tile nextTile = aTile.next();
		
		// can make left chi?
		if (aTile.getOrder() <= 7 && 
		    hand.getClosedTileIdx(nextTile) >= 0 && 
		    hand.getClosedTileIdx(nextTile.next()) >= 0)
		    return Action.LEFT_CHI;
		
		// can make center chi?
		if (aTile.getOrder() <= 8 &&
		    aTile.getOrder() >= 2 &&
		    hand.getClosedTileIdx(prevTile) >= 0 &&
		    hand.getClosedTileIdx(nextTile) >= 0)
		    return Action.CENTER_CHI;
		
		// can make right chi?
		if (aTile.getOrder() >= 3 &&
		    hand.getClosedTileIdx(prevTile) >= 0 &&
		    hand.getClosedTileIdx(prevTile.previous()) >= 0)
		    return Action.RIGHT_CHI;	
		return Action.NONE;
	    }
	    else 
		return Action.NONE;	
	}
    }

    private void testSets()
    {
	int i;
	Tile aTile;

	for ( i = 0; i < closedTiles.size()-2; )
	{
	   aTile = (Tile) closedTiles.get(i);
	   if (aTile.equals((Tile) closedTiles.get(i+2)))
	       i+=3;
	   else
	       if (aTile.equals((Tile) closedTiles.get(i+1)))
	       {
		   aTile.addPoints(LEVEL2);
		   ((Tile) closedTiles.get(i+1)).addPoints(LEVEL2);
		   i+=2;
	       }
	       else
	       {
		   aTile.addPoints(LEVEL3);
		   i++;
	       }
	}
	if (i == closedTiles.size()-1)
	    ((Tile) closedTiles.get(i)).addPoints(LEVEL3);
	else
	    if (i == closedTiles.size()-2)
		if (((Tile) closedTiles.get(i)).equals((Tile) closedTiles.get(i+1)))
		{
		    ((Tile) closedTiles.get(i)).addPoints(LEVEL2);
		    ((Tile) closedTiles.get(i+1)).addPoints(LEVEL2);
		}
		else
		{
		    ((Tile) closedTiles.get(i)).addPoints(LEVEL3);
		    ((Tile) closedTiles.get(i+1)).addPoints(LEVEL3);
		}
    }

    private void testRuns()
    {
	int i;
	Tile aTile;

	for ( i = 0; i < closedTiles.size(); i++)
	{
	    aTile = (Tile) closedTiles.get(i);
	    if (aTile.isHonor())
		continue;
	    if(aTile.getOrder() == 1)
	    {
		if ((hand.getClosedTileIdx(aTile.next()) > 0) &&
		    (hand.getClosedTileIdx(aTile.next().next()) > 0))
		    continue;
		else 
		    if (hand.getClosedTileIdx(aTile.next()) > 0)
			aTile.addPoints(LEVEL2);
		    else
			aTile.addPoints(LEVEL3);
		continue;
	    }
	    if(aTile.getOrder() == 9)
	    {
		if ((hand.getClosedTileIdx(aTile.previous()) > 0) &&
		    (hand.getClosedTileIdx(aTile.previous().previous()) > 0))
		    continue;
		else 
		    if (hand.getClosedTileIdx(aTile.previous()) > 0)
			aTile.addPoints(LEVEL2);
		    else
			aTile.addPoints(LEVEL3);
		continue;
	    }
	    if (((hand.getClosedTileIdx(aTile.previous()) > 0) && 
		 (hand.getClosedTileIdx(aTile.next()) > 0)) || 
		((hand.getClosedTileIdx(aTile.previous()) > 0) &&
		 (hand.getClosedTileIdx(aTile.previous().previous()) > 0)) ||
		((hand.getClosedTileIdx(aTile.next()) > 0) &&
		 (hand.getClosedTileIdx(aTile.next().next()) > 0)))
		continue;
	    else
		if ((hand.getClosedTileIdx(aTile.previous()) > 0) ||
		    (hand.getClosedTileIdx(aTile.next()) > 0))
		    aTile.addPoints(LEVEL2);
		else
		    aTile.addPoints(LEVEL3);
	}
    }

    private void testHonor() 
    {
    }

    
    
    private static final int LEVEL1 = 10;
    private static final int LEVEL2 = 20;
    private static final int LEVEL3 = 30;
    private static final int LEVEL4 = 40;
    private Random rand = new Random();
}

