package mahjong;

import java.util.*;
import javax.swing.*;

public class Table extends Thread
{
    public static Table getInstance()
    {
	return (theTable);
    }

    public GameInterface getGameInterface()
    {
	return (gInterface);
    }

    public short getCurrentRound()
    {
	return (currentRound);
    }

    public Tile[] getDoras()
    {
	int first = (NUM_TILES - 1) - NUM_DEAD_TILES;
	int last = nextDoraIdx;

	System.out.println("first = " + first);
	System.out.println("last  = " + last);

	Tile[] result = new Tile[last - first];
	for (int i = first, j = 0; i < last; i++, j++) result[j] = wallTiles[i];

	return (result);
    }

    private Table()
    {
	// initializing players
	players = new Player[4];
	players[0] = new RealPlayer(Position.EAST);
	players[1] = new VirtualPlayer(Position.SOUTH);
	players[2] = new VirtualPlayer(Position.WEST);
	players[3] = new VirtualPlayer(Position.NORTH);

	// generating tiles
	wallTiles = new Tile[NUM_TILES];
	int i, j = 0;
	for (int k = 0; k < 4; k++)
	{
	    for (i = 1; i <= 9; i++, j++) 
		wallTiles[j] = new Tile(TileType.PINZU, (short)i);

	    for (i = 1; i <= 9; i++, j++)
		wallTiles[j] = new Tile(TileType.SOUZU, (short)i);

	    for (i = 1; i <= 9; i++, j++) 
		wallTiles[j] = new Tile(TileType.WANZU, (short)i);

	    for (i = 1; i <= 4; i++, j++) 
		wallTiles[j] = new Tile(TileType.KAZEHAI, (short)i);

	    for (i = 1; i <= 3; i++, j++) 
		wallTiles[j] = new Tile(TileType.SANGENPAI, (short)i);
	}

	// filling the scoreTable in
	scoreTable.put("" + new ScoreKey( 20,  1), new ScoreValue( 1000,  500,   300,  1500,   500));
	scoreTable.put("" + new ScoreKey( 30,  1), new ScoreValue( 1000,  500,   300,  1500,   500));
	scoreTable.put("" + new ScoreKey( 40,  1), new ScoreValue( 1300,  700,   400,  2000,   700));
	scoreTable.put("" + new ScoreKey( 50,  1), new ScoreValue( 1600,  800,   400,  2400,   800));
	scoreTable.put("" + new ScoreKey( 60,  1), new ScoreValue( 2000, 1000,   500,  2900,  1000));
	scoreTable.put("" + new ScoreKey( 70,  1), new ScoreValue( 2300, 1200,   600,  3400,  1200));
	scoreTable.put("" + new ScoreKey( 80,  1), new ScoreValue( 2600, 1300,   700,  3900,  1300));
	scoreTable.put("" + new ScoreKey( 90,  1), new ScoreValue( 2900, 1500,   800,  4400,  1500));
	scoreTable.put("" + new ScoreKey(100,  1), new ScoreValue( 3200, 1600,   800,  4800,  1600));
	scoreTable.put("" + new ScoreKey(110,  1), new ScoreValue( 3600, 1800,   900,  5300,  1800)); 

	scoreTable.put("" + new ScoreKey( 20,  2), new ScoreValue( 1300,  700,   400,  2000,   700));
	scoreTable.put("" + new ScoreKey( 30,  2), new ScoreValue( 2000, 1000,   500,  2900,  1000));
	scoreTable.put("" + new ScoreKey( 40,  2), new ScoreValue( 2600, 1300,   700,  3900,  1300));
	scoreTable.put("" + new ScoreKey( 50,  2), new ScoreValue( 3200, 1600,   800,  4800,  1600));
	scoreTable.put("" + new ScoreKey( 60,  2), new ScoreValue( 3900, 2000,  1000,  5800,  2000));
	scoreTable.put("" + new ScoreKey( 70,  2), new ScoreValue( 4500, 2300,  1200,  6800,  2300));
	scoreTable.put("" + new ScoreKey( 80,  2), new ScoreValue( 5200, 2600,  1300,  7700,  2600));
	scoreTable.put("" + new ScoreKey( 90,  2), new ScoreValue( 5800, 2900,  1500,  8700,  2900));
	scoreTable.put("" + new ScoreKey(100,  2), new ScoreValue( 6400, 3200,  1600,  9600,  3200));
	scoreTable.put("" + new ScoreKey(110,  2), new ScoreValue( 7100, 3600,  1800, 10600,  3600));

	scoreTable.put("" + new ScoreKey( 20,  3), new ScoreValue( 2600, 1300,   700,  3900,  1300));
	scoreTable.put("" + new ScoreKey( 30,  3), new ScoreValue( 3900, 2000,  1000,  5800,  2000));
	scoreTable.put("" + new ScoreKey( 40,  3), new ScoreValue( 5200, 2600,  1300,  7700,  2600));
	scoreTable.put("" + new ScoreKey( 50,  3), new ScoreValue( 6400, 3200,  1600,  9600,  3200));
	scoreTable.put("" + new ScoreKey( 60,  3), new ScoreValue( 7700, 3900,  2000, 11600,  3900));
	scoreTable.put("" + new ScoreKey( 70,  3), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey( 80,  3), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey( 90,  3), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey(100,  3), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey(110,  3), new ScoreValue( 8000, 4000,  2000, 12000,  4000));

	scoreTable.put("" + new ScoreKey( 20,  4), new ScoreValue( 5200, 2600,  1300,  7700,  2600));
	scoreTable.put("" + new ScoreKey( 30,  4), new ScoreValue( 7700, 3900,  2000, 11600,  3900));
	scoreTable.put("" + new ScoreKey( 40,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey( 50,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey( 60,  4), new ScoreValue( 8000, 4000,  2000, 11600,  4000));
	scoreTable.put("" + new ScoreKey( 70,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey( 80,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey( 90,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey(100,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey(110,  4), new ScoreValue( 8000, 4000,  2000, 12000,  4000));
	
	scoreTable.put("" + new ScoreKey(  0,  5), new ScoreValue( 8000,  4000, 2000, 12000,  4000));
	scoreTable.put("" + new ScoreKey(  0,  6), new ScoreValue(12000,  6000, 3000, 18000,  6000));
	scoreTable.put("" + new ScoreKey(  0,  7), new ScoreValue(12000,  6000, 3000, 18000,  6000));
	scoreTable.put("" + new ScoreKey(  0,  8), new ScoreValue(16000,  8000, 4000, 24000,  8000));
	scoreTable.put("" + new ScoreKey(  0,  9), new ScoreValue(16000,  8000, 4000, 24000,  8000));
	scoreTable.put("" + new ScoreKey(  0, 10), new ScoreValue(16000,  8000, 4000, 24000,  8000));
	scoreTable.put("" + new ScoreKey(  0, 11), new ScoreValue(24000, 12000, 6000, 36000, 12000));
	scoreTable.put("" + new ScoreKey(  0, 12), new ScoreValue(24000, 12000, 6000, 36000, 12000));
	scoreTable.put("" + new ScoreKey(  0, 13), new ScoreValue(32000, 16000, 8000, 48000, 16000));

	// initializing the game interface;
	gInterface = new GameInterface();

    }

    private void nap()
    {
	// makes this thread take a nap
	try { Thread.sleep(NAP_TIME); }
	catch (InterruptedException e) {}
    }

    private void clearHands()
    {
	for (int i = 0; i < players.length; i++) players[i].reset();
    }

    private void shuffle()
    {
	Random rand = new Random();
	Tile auxTile;
	int j, k;

	// exchange tiles randomly 100 times
	for (int i = 0; i < 100; i++)
	{
	    j = Math.abs(rand.nextInt()%wallTiles.length);
	    k = Math.abs(rand.nextInt()%wallTiles.length);

	    auxTile = wallTiles[j];
	    wallTiles[j] = wallTiles[k];
	    wallTiles[k] = auxTile;
	}
    }

    private void distribute()
    {
	// give aech player 13 tiles
	for (int i = 0; i < 52; i += 4)
	{
	    players[0].getHand().addClosedTile(wallTiles[i + 0]);
	    players[1].getHand().addClosedTile(wallTiles[i + 1]);
	    players[2].getHand().addClosedTile(wallTiles[i + 2]);
	    players[3].getHand().addClosedTile(wallTiles[i + 3]);
	}
	nextTileIdx = 52;
    }

    private Tile selectNextDora()
    {
	// no more than 5 doras.
	if (nextDoraIdx >= ((NUM_TILES - 1) - NUM_DEAD_TILES) + NUM_DORAS)
	{
	    System.out.println("selectNextDora() == null!");
	    return (null);
	}
	else
	{
	    System.out.println("selectNextDora() == " + wallTiles[nextDoraIdx]);
	    return (wallTiles[nextDoraIdx++]);
	}
    }

    private void initGame()
    {
	// define new game state
	currentDealer  = Position.EAST;
	currentPlayer  = Position.EAST;
	currentRound   = Position.EAST;
	currentStage   = (short)1;
	nDrawHands     = 0;
	nFinishedHands = 0;
	
	// cleaning score
	for (int i = 0; i < players.length; i++) players[i].setScore(25000);
    }

    private void initHand()
    {
	// setup tiles
	clearHands();
	shuffle();
	distribute();
	nextDoraIdx = (NUM_TILES - 1) - NUM_DEAD_TILES;
	Tile dora = selectNextDora();
	
	// define the first move
	lastPlayer = -1;
	currentPlayer = currentDealer;
	currentTile = wallTiles[nextTileIdx++];
	action = Action.DRAW;
	discardAction = Action.NONE;

	// nobody have finished a hand yet
	Result.getInstance().reset();

	// notify the gameInterface
	Hand[] hands = new Hand[players.length];
	for (int i = 0; i < players.length; i++) hands[i] = players[i].getHand();
	gInterface.notifyInit(currentDealer, currentRound, hands);
	gInterface.notifyRemainingTiles((short)numTilesOnTheWall());
	gInterface.notifyDora(dora);
	nap();
    }

    private int numTilesOnTheWall()
    {
	return (((NUM_TILES-1) - NUM_DEAD_TILES) - nextTileIdx);
    }

    private void processFinishedHand()
    {
	int tali = 0;
	int winner = currentPlayer;
	
	// defining init tali
	if (action == Action.RON && players[winner].getHand().isClosed())
	{
	    tali = CLOSED_RON_INIT_TALI;
	}
	else
	{
	    tali = OPEN_TSUMO_INIT_TALI;
	}

	// counting tali for pon's and kan's...
	// ... lokking for open sets
	OpenMeld m;
	for (int i = 0; i < players[winner].getHand().getOpenPon().size(); i++)
	{
	    m = (OpenMeld)players[winner].getHand().getOpenPon().get(i);
	    if (m.getStolenTile().isSimple()) 
		tali += OPEN_SIMPLE_SET_TALI;
	    else
		tali += OPEN_END_SET_TALI;
	}

	// ...looking for open kans
	for (int i = 0; i < players[winner].getHand().getOpenKan().size(); i++)
	{
	    m = (OpenMeld)players[winner].getHand().getOpenKan().get(i);
	    if (m.getStolenTile().isSimple()) 
		tali += OPEN_SIMPLE_KAN_TALI;
	    else
		tali += OPEN_END_KAN_TALI;
	}

	// ...looking for closed kans
	for (int i = 0; i < players[winner].getHand().getClosedKan().size(); i++)
	{
	    m = (OpenMeld)players[winner].getHand().getClosedKan().get(i);
	    if (m.getTiles()[0].isSimple()) 
		tali += CLOSED_SIMPLE_KAN_TALI;
	    else
		tali += CLOSED_END_SET_TALI;
	}

	// counting tali for closed tiles's sets
	Tile theTile;
	int nSame;
	for (int i = 0, j; i < players[winner].getHand().getClosedTiles().size(); i += nSame)
	{
	    theTile = (Tile)players[winner].getHand().getClosedTiles().get(i);
	    nSame = 0;
	    j = i;
	    while (j < players[winner].getHand().getClosedTiles().size() && 
		   theTile.equals((Tile)players[winner].getHand().getClosedTiles().get(j)))
	    {
		nSame++;
		j++;
	    }

	    if (nSame == 3)
	    {
		if (theTile.isSimple()) 
		    tali += CLOSED_SIMPLE_SET_TALI;
		else
		    tali += CLOSED_END_SET_TALI;

	    }
	    else if (nSame == 2)
	    {
		if (theTile.isColor() ||
		    (theTile.isWind() && (theTile.getOrder() == currentRound || 
					  theTile.getOrder() == players[winner].getPosition())))
		    tali += VALUE_HONOR_HEAD_TALI;
	    }
	}

	// rounding up tali
	tali = tali + ((10 - (tali % 10)) %10);

	// calculating points
	int totalPoints = 0;
	int totalHan = Result.getInstance().getTotalHan();
	ScoreValue value = (ScoreValue)scoreTable.get("" + new ScoreKey(totalHan >  4 ?  0 : tali, 
									totalHan > 13 ? 13 : totalHan));
	if (winner == currentDealer)
	{
	    if (action == Action.RON)
	    {
		totalPoints += value.dealerRon;
		players[winner].setScore(players[winner].getScore() + value.dealerRon);
		players[lastPlayer].setScore(players[lastPlayer].getScore() - value.dealerRon);
	    }
	    else
            {
		for (int j = 0; j < players.length; j++)
		    if (j == winner)
		    {
			totalPoints += 3*value.dealerTsumo;
			players[j].setScore(players[j].getScore() + 3*value.dealerTsumo);
		    }
		    else
			players[j].setScore(players[j].getScore() - value.dealerTsumo);
		
	    }
	}
	else
	{
	    if (action == Action.RON)
	    {
		totalPoints += value.nonDealerRon;
		players[winner].setScore(players[winner].getScore() + value.nonDealerRon);
		players[lastPlayer].setScore(players[lastPlayer].getScore() - value.nonDealerRon);
	    }
	    else
            {
		for (int j = 0; j < players.length; j++)
		{
		    if (j == winner)
		    {
			totalPoints += value.nonDealerFromDealer + 2*value.nonDealerFromNonDealer;
			players[j].setScore(players[j].getScore() + 
					    value.nonDealerFromDealer + 
					    2 * value.nonDealerFromNonDealer);
		    }
		    else
			if (j == currentDealer)
			    players[j].setScore(players[j].getScore() - value.nonDealerFromDealer);
			else
			    players[j].setScore(players[j].getScore() - value.nonDealerFromNonDealer);
		}
	    }	    
	}

	// adding accumulated reach points
	for (int i = 0; i < players.length; i++) 
	    if (i != winner && players[i].isReach()) 
	    {
		reachPoints += 1000;
		players[i].setScore(players[i].getScore()-1000);
	    }
	totalPoints += reachPoints;
	players[winner].setScore(players[winner].getScore() + reachPoints);
	reachPoints = 0;

	// getting players' score
	int[] score = new int[4];
	for (int i = 0; i < players.length; i++) score[i] = players[i].getScore();

	// showing the finished hand dialog
	gInterface.invokeFinishedHandDialog(players[currentPlayer], score, tali, totalPoints);
	
	// setting up the next stage
	nDrawHands = 0;
	if (winner != currentDealer)
	{
	    currentDealer = Position.nextPosition(currentDealer);
	    currentStage++;
	    if (currentStage >= 5) currentRound  = Position.nextPosition(currentRound);
	    nFinishedHands = 0;
	}
	else nFinishedHands++;
    }

    private void processDrawHand()
    {
	// calculating tenpai points
	boolean[] tenpai = new boolean[4];
	int nTenpai = 0;
	for (int i = 0; i < 4; i++)
	{
	    tenpai[i] = players[i].isTenpai();
	    if (tenpai[i]) nTenpai++;
	}

	for (int i = 0; i < 4; i++)
	{
	    switch(nTenpai)
	    {
	    case 1:
		if (tenpai[i])
		    players[i].setScore(players[i].getScore() + 3000);
		else
		    players[i].setScore(players[i].getScore() - 1000);
		break;

	    case 2:
		if (tenpai[i])
		    players[i].setScore(players[i].getScore() + 1500);
		else
		    players[i].setScore(players[i].getScore() - 1500);

	    case 3:
		if (tenpai[i])
		    players[i].setScore(players[i].getScore() + 1000);
		else
		    players[i].setScore(players[i].getScore() - 3000);
	    }
	}

	// adding reach accumulated points
	for (int i = 0; i < players.length; i++)
	    if (players[i].isReach())
	    {
		players[i].setScore(players[i].getScore() - 1000);
		reachPoints += 1000;
	    }
	
	// getting players' score
	int[] score = new int[4];
	for (int i = 0; i < players.length; i++) score[i] = players[i].getScore();

	// showing the finished hand dialog
	FinishedHandDialog dialog = new FinishedHandDialog(gInterface);
	dialog.show(null, score, 0, 0);

	// defining next stage
	if (currentRound == Position.EAST && !players[currentDealer].isTenpai())
	{
	    currentStage++;
	    if (currentStage >=5 ) currentRound = Position.nextPosition(currentRound);
	    currentDealer = Position.nextPosition(currentDealer);
	    nDrawHands = 0;
	    nFinishedHands = 0;
	}
	else
	{
	    nDrawHands++;
	    nFinishedHands = 0;
	}
    }

    private void setNextStep()
    {
	if (discardAction == Action.CLOSED_KAN ||
	    discardAction == Action.OPEN_KAN ||
	    discardAction == Action.LATE_KAN)
	{
	    Tile dora = selectNextDora();
	    if (dora != null) 
	    {
		gInterface.notifyDora(dora);
		nap();
	    }
	    action = Action.DRAW;
	    currentTile = wallTiles[nextTileIdx++];
	    return;
	}

	// currentTile == null => someone finished a hand
	if (currentTile == null) return;

	int[] actions = new int[3];
	short[] positions = new short[3];
	short player; 

	lastPlayer = currentPlayer;
	// ask player if they want the last discarded tiles
	player = Position.nextPosition(currentPlayer);
	positions[0] = player;
	actions[0] = players[player].wantTile(currentTile, StolenPlayer.LEFT);

	player = Position.nextPosition(player);
	positions[1] = player;
	actions[1] = players[player].wantTile(currentTile, StolenPlayer.ACROSS);

	player = Position.nextPosition(player);
	positions[2] = player;
	actions[2] = players[player].wantTile(currentTile, StolenPlayer.RIGHT);
	
	// check the priority of the requests
	int i;
	for (i = 0; i < 3; i++)
	{
	    if (actions[i] == Action.RON)
	    {
		action = actions[i];
		currentPlayer = positions[i];
		players[lastPlayer].getHand().removeDiscard();
		gInterface.notifyPlayerChanged(lastPlayer, players[lastPlayer].getHand());
		return;
	    }
	}

	for (i = 0; i < 3; i++)
	{
	    if (actions[i] == Action.LATE_KAN ||
		actions[i] == Action.OPEN_KAN ||
		actions[i] == Action.CLOSED_KAN)
	    {
		action = actions[i];
		currentPlayer = positions[i];
		players[lastPlayer].getHand().removeDiscard();
		gInterface.notifyPlayerChanged(lastPlayer, players[lastPlayer].getHand());
		return;
	    }
	}

	for (i = 0; i < 3; i++)
	{
	    if (actions[i] == Action.PON)
	    {
		action = actions[i];
		currentPlayer = positions[i];
		players[lastPlayer].getHand().removeDiscard();
		gInterface.notifyPlayerChanged(lastPlayer, players[lastPlayer].getHand());
		return;
	    }
	}

	for (i = 0; i < 3; i++)
	{
	    if (actions[i] == Action.LEFT_CHI   ||
		actions[i] == Action.CENTER_CHI ||
		actions[i] == Action.RIGHT_CHI)
	    {
		action = actions[i];
		currentPlayer = positions[i];
		players[lastPlayer].getHand().removeDiscard();
		gInterface.notifyPlayerChanged(lastPlayer, players[lastPlayer].getHand());
		return;
	    }
	}

	// nobody wants the discarded tile
	action = Action.DRAW;
	currentPlayer = Position.nextPosition(currentPlayer);
	currentTile = wallTiles[nextTileIdx++];
	return;
    }

    public void run()
    {
	// while player don't quit
	while(true)
	{
	    // start new game?
	    Object[] options = {"Jnior", "Snior", "Master"};
	    int n = JOptionPane.showOptionDialog(gInterface,
						 "Escolha o nvel de dificuldade:",
						 "Novo Jogo",
						 JOptionPane.YES_NO_CANCEL_OPTION,
						 JOptionPane.QUESTION_MESSAGE,
						 null,
						 options,
						 options[0]);
	    //if (n != JOptionPane.YES_OPTION) break;
	    switch (n)
            {
	    case JOptionPane.YES_OPTION:
	    {
		System.out.println("Junior Level...");
		((VirtualPlayer)players[Position.SOUTH]).setIntelligence(new Junior(players[Position.SOUTH]));
		((VirtualPlayer)players[Position.WEST]).setIntelligence(new Junior(players[Position.WEST]));
		((VirtualPlayer)players[Position.NORTH]).setIntelligence(new Junior(players[Position.NORTH]));		
		break;
	    }
	    case JOptionPane.NO_OPTION:
	    {
		System.out.println("Senior Level...");
		((VirtualPlayer)players[Position.SOUTH]).setIntelligence(new Senior(players[Position.SOUTH]));
		((VirtualPlayer)players[Position.WEST]).setIntelligence(new Senior(players[Position.WEST]));
		((VirtualPlayer)players[Position.NORTH]).setIntelligence(new Senior(players[Position.NORTH]));
		break;
	    }
	    case JOptionPane.CANCEL_OPTION:
            {
		System.out.println("Master Level...");
		((VirtualPlayer)players[Position.SOUTH]).setIntelligence(new Master(players[Position.SOUTH]));
		((VirtualPlayer)players[Position.WEST]).setIntelligence(new Master(players[Position.WEST]));
		((VirtualPlayer)players[Position.NORTH]).setIntelligence(new Master(players[Position.NORTH]));
		break;
	    }
	    } 
	    
	    // new game
	    initGame();
	    initHand();
	    while (currentRound != Position.WEST)
            {
		if (currentStage >= 5) currentStage = 1;
		while (currentStage < 5)
		{
		    // notify who is the current player
		    gInterface.notifyWhoIsPlaying(currentPlayer);
		    nap();
		    
		    if (Result.getInstance().getStatus())
		    {
			// player have finished a hand
			processFinishedHand();
			initHand();
			break;
		    }
		
		    if (numTilesOnTheWall() < 0)
		    {
			// no more tiles avaiable: draw hand
			processDrawHand();
			initHand();
			break;
		    }
		    
		    // nobody finished a hand and still have tiles on the wall
		    players[currentPlayer].receiveTile(currentTile, action, lastPlayer);
		    gInterface.notifyPlayerChanged(currentPlayer, players[currentPlayer].getHand());
		    gInterface.notifyRemainingTiles((short)numTilesOnTheWall());
		    nap();

		    // if last player action == OPEN_KAN must play again
		    if (action == Action.OPEN_KAN)
		    {
			Tile dora = selectNextDora();
			if (dora != null)
			{
			    gInterface.notifyDora(dora);
			    nap();
			}
			currentTile = wallTiles[nextTileIdx++];
			action = Action.DRAW;
			continue;
		    }

		    // if last player action == RON no need to continue
		    if (action == Action.RON) 
                    {
			continue;
		    }

		    // nothing different happened...
		    currentTile = players[currentPlayer].discard(action);
		    discardAction = players[currentPlayer].getLastAction();		    
		    if (discardAction == Action.REACH)
		    {
			gInterface.notifyReach(currentPlayer, 
					       (short)(players[currentPlayer].getHand().getDiscardedTiles().size()-1));
		    }
		    gInterface.notifyPlayerChanged(currentPlayer, players[currentPlayer].getHand());
		    nap();
		    setNextStep();
		}
	    }
	}
    }

    // private data
    private static Table theTable = new Table();
    private GameInterface gInterface;
    private Player[] players;
    private HashMap scoreTable = new HashMap();

    private short currentDealer;
    private short currentPlayer;
    private short currentRound;
    private short currentStage;
    private short lastPlayer;

    private Tile currentTile;
    private Tile[] wallTiles;
    
    private int action;
    private int discardAction;
    private int nextTileIdx;
    private int nextDoraIdx;
    private int nDrawHands;
    private int nFinishedHands;
    private int reachPoints;

    // some constants
    public static final int NUM_TILES      = 136;
    public static final int NUM_DORAS      =   5;
    public static final int NUM_DEAD_TILES =  14;
    public static final int NAP_TIME       = 200;
    
    // tali scoring constants
    public static final int CLOSED_RON_INIT_TALI   = 30;
    public static final int OPEN_TSUMO_INIT_TALI   = 20;
    public static final int CLOSED_SIMPLE_SET_TALI =  4;
    public static final int OPEN_SIMPLE_SET_TALI   =  2;
    public static final int CLOSED_END_SET_TALI    =  8;
    public static final int OPEN_END_SET_TALI      =  4;
    public static final int CLOSED_SIMPLE_KAN_TALI = 16;
    public static final int OPEN_SIMPLE_KAN_TALI   =  8;
    public static final int CLOSED_END_KAN_TALI    = 32;
    public static final int OPEN_END_KAN_TALI      = 16;
    public static final int VALUE_HONOR_HEAD_TALI  =  2;
}

class ScoreKey
{
    public ScoreKey(int t, int h)
    {
	tali = t;
	han = h;
    }

    public String toString()
    {
	return ("" + tali + "_" + han);
    }

    // points
    public int tali;
    public int han;
}

class ScoreValue
{
    public ScoreValue(int ndr, int ndfd, int ndfnd, int dr, int dt)
    {
	nonDealerRon = ndr;
	nonDealerFromDealer = ndfd;
	nonDealerFromNonDealer = ndfnd;
	dealerRon = dr;
	dealerTsumo = dt;
    }
    
    // points earned for non dealer winner
    public int nonDealerRon;
    public int nonDealerFromDealer;
    public int nonDealerFromNonDealer;

    // points earned for dealer winner;
    public int dealerRon;
    public int dealerTsumo;
}




