package mahjong;

import java.util.*;

public class Tile
{
    public Tile(short type, short order)
    {
	this.type = type;
	this.order = order;
	this.name = getName(type, order);
	this.points = 0;
    }

    public void printTest()
    {
	System.out.println("name........: " + name);
	System.out.println("type........: " + TileType.name(type));
	System.out.println("order.......: " + order);
	System.out.println("isTerminal..: " + isTerminal());
	System.out.println("isWind......: " + isWind());
	System.out.println("isColor.....: " + isColor());
	System.out.println("isHonor.....: " + isHonor());
	System.out.println("isEnd.......: " + isEnd());
	System.out.println("isSimple....: " + isSimple());
	System.out.println("----------------------------");
    }

    public static String getName(short type, short order)
    {
	if (type == TileType.PINZU)
	{
	    switch (order)
	    {
	    case 1: return "iipin";
	    case 2: return "ryanpin";
            case 3: return "sanpin";
	    case 4: return "suupin";
	    case 5: return "uupin";
	    case 6: return "ryuupin";
	    case 7: return "chiipin";
	    case 8: return "paapin";
	    case 9: return "chuupin";
	    default: return "unkown tile";
	    }
	}
	else if (type == TileType.SOUZU)
	{
	    switch (order)
            {
	    case 1: return "iisou";
	    case 2: return "ryanzou";
	    case 3: return "sanzou";
	    case 4: return "sussou";
	    case 5: return "uusou";
	    case 6: return "ryuusou";
	    case 7: return "chiisou";
	    case 8: return "passou";
	    case 9: return "chuusou";
	    default: return "unkown tile";
	    }
	}
	else if (type == TileType.WANZU)
	{
	    switch (order)
            {
	    case 1: return "iiwan";
	    case 2: return "ryanwan";
	    case 3: return "sanwan";
	    case 4: return "suuwan";
	    case 5: return "uuwan";
	    case 6: return "ryuuwan";
	    case 7: return "chiiwan";
	    case 8: return "paawan";
	    case 9: return "chuuwan";
	    default: return "unkown tile";
	    }
	}
	else if (type == TileType.KAZEHAI)
	{
	    switch (order)
            {
	    case 1: return "ton";
	    case 2: return "nan";
	    case 3: return "shaa";
	    case 4: return "pei";
	    default: return "unkown tile";
	    }
	}
	else if (type == TileType.SANGENPAI)
	{
	    switch (order)
            {
	    case 1: return "haku";
	    case 2: return "hatsu";
	    case 3: return "chun";
	    default: return "unkown tile";
	    }
	}
	else return "unkown tile";
    }

    public boolean isTerminal() 
    {
	return ((type  == TileType.PINZU || type == TileType.SOUZU || type == TileType.WANZU) &&
		(order == 1 || order == 9));
    }

    public boolean isWind()
    {
	return (type == TileType.KAZEHAI);
    }

    public boolean isColor()
    { 
	return (type == TileType.SANGENPAI);
    }

    public boolean isHonor()
    {
	return (isWind() || isColor());
    }

    public boolean isEnd()
    {
	return (isTerminal() || isWind() || isColor());
    }

    public boolean isSimple()
    {
	return (!isEnd());
    }

    public String getName()
    {
	return (name);
    }

    public short getType()
    {
	return (type);
    }

    public short getOrder()
    {
	return (order);
    }

    public String toString()
    {
	return (name);
    }
    
    public void initPoints()
    {
	points = 0;
    }

    public void addPoints(int value)
    {
	points+= value;
    }
    
    public int getPoints()
    {
	return points;
    }

    public Tile next()
    {
	short newOrder;
	switch(type)
	{
	case TileType.PINZU:
	case TileType.SOUZU:
	case TileType.WANZU:
	    newOrder = (short)(order + 1);
	    if (newOrder > 9) newOrder = 1;
	    return (new Tile(type, newOrder));
	case TileType.KAZEHAI:
	    newOrder = (short)(order + 1);
	    if (newOrder > 4) newOrder = 1;
	    return (new Tile(type, newOrder));
	case TileType.SANGENPAI:
	    newOrder = (short)(order + 1);
	    if (newOrder > 3) newOrder = 1;
	    return (new Tile(type, newOrder));
	default: return (null);
	}
    }

    public Tile previous()
    {
	short newOrder;
	switch(type)
	{
	case TileType.PINZU:
	case TileType.SOUZU:
	case TileType.WANZU:
	    newOrder = (short)(order - 1);
	    if (newOrder < 1) newOrder = 9;
	    return (new Tile(type, newOrder));
	case TileType.KAZEHAI:
	    newOrder = (short)(order - 1);
	    if (newOrder < 1) newOrder = 4;
	    return (new Tile(type, newOrder));
	case TileType.SANGENPAI:
	    newOrder = (short)(order - 1);
	    if (newOrder < 1) newOrder = 3;
	    return (new Tile(type, newOrder));
	default: return (null);
	}	
    }
    
    public boolean equals (Object anObject)
    {
	if (anObject == null) return (false);
	if (anObject instanceof Tile) return (((Tile)anObject).type == type && ((Tile)anObject).order == order);
	return (false);
    }
   
    private String name;
    private short  type;
    private short  order;
    private int    points;
}































